/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Four11;
import java.lang.*;
import java.io.*;
import java.util.*;

/**
  * This class reads the configuration file and maintains the topology 
  * of the network. The topology is maintained in a lisp-like list format.
  * 
  * When a request arrives the <tt>MessageHandler</tt>
  * thread forwards the request to this class using the <i><tt>get()</tt></i>
  * interface. The request itself is in a list format with the requested quantity
  * indicated by a wildcard (<tt>*</tt>) symbol. The request is then matched
  * against the internal list and the list/value matching the wildcard is
  * returned as the reply.
  * 
  * @see MessageHandler
  */
public class AssocList {
  private Tokenizer ft;
  private List assocList;
  private transient Token tok;
  private static int TOK_EOM = 0x0;
  private static int TOK_WORD = 0x3;
  private Vector bfsQ;

  /**
    * initializes the object with the name of the network to configure
    * The name of the configuration file to read is derived by concatenating
    * ".conf" to the network name.
    *
    * @param netName the network name
  */
  public AssocList(String netName) {
    try {
      // open config file
      bfsQ = null;
      FileInputStream fis = new FileInputStream(netName + ".conf");

      ft = new Tokenizer(fis);
      ft.setCharAsByte(true);
      assocList = null;
      tok = ft.nextToken();
      do {
        assocList = addToList(assocList);
      } while (tok.type() != TOK_EOM);
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }


  private List addToList(List l) {
    tok = ft.nextToken();
    if (tok.value().equals("(")) {
      if (l == null) l = new List();
       l.setCar(addToList((List) l.car()));
       l = l.setCdr(addToList(l.cdr()));
    }
    else if (tok.type() == TOK_WORD) {
      if (l == null) l = new List();
      l.setCar(tok.value());
      l = l.setCdr(addToList(l.cdr()));
    }
    else if (tok.value().equals(")")) {
      l = null;
    }
  
    return l;
  }
  
  private void print(Object o, StringWriter out) {
    if (o == null) return;
    
    if (o.getClass().getName().equals("magician.Four11.List")) {
      List l = (List) o;

      if(l.car().getClass().getName().equals("magician.Four11.List")) {

        out.write("(");
        print(l.car(), out);
        out.write(")");
      }
      else
        out.write(" "+ l.car());
      print(l.cdr(), out);

    }
    else {
      out.write("" + o);
    }
      
  }

    /**
      * returns reply to the query sent by the active node
      * The reply can be a list or a value
      *
      * @param s the request message
      * @return the reply string
      */
    public synchronized String get(String s) {
    // send string to a string tokenizer

    StringTokenizer st = new StringTokenizer(s, " ()\n\t;");
    StringWriter sw = new StringWriter();
    //PrintWriter pw = new PrintWriter(System.out, true);
    List selectors = tokenize(st);

    try {
      List subtree = getSublist(selectors);
    
      if (subtree == assocList) {
        sw.write("(");
        print(subtree, sw);
        sw.write(")");  
      } else if (subtree != null) {
        if (subtree.car().getClass().getName().equals("magician.Four11.List")) {
          sw.write("(");
          print(subtree.car(), sw);
          sw.write(")");
        } else
          print(subtree.car(), sw);
      } else {
      sw.write("()");
      }
    } catch (NoSuchElementException e) {
      sw.write("()");
    }
      
    sw.flush();
    // System.err.println("Request = " + s);
    // System.err.println(" Response = " + sw.toString());
    return sw.toString();
  }
  
  private List getSublist(List l) {
    // run BFS on assocList with the token list obtained
    List subtree = assocList;
    while(subtree != null || l != null) {
      if (l.car().equals("*")) 
        break;
    
      bfsQ = new Vector();
      subtree = bfs(subtree, l.car());
      l = l.cdr();
    }
    return subtree;
  }
  
  private List tokenize(StringTokenizer st) {
    List l = null;
    if (st.hasMoreTokens()) {
      l = new List();
      l.setCar(st.nextToken());
      l = l.setCdr(tokenize(st));
    }
    else
      l = null;
    
    return l;
  }
    
    
  private List bfs(List t, Object key) {
    // StringWriter debug = new StringWriter();
    // print(t, debug);
    // System.err.println("BFS: key = " + key + " List = " + debug.toString());
    if (t == null) return null;
    List l = null;
    
    if (t.car().getClass().getName().equals("magician.Four11.List")) {
      // put car of list in queue to open later
      bfsQ.addElement(t.car());
    }
    else if (t.car().equals(key)) {
      return t.cdr();
    }
    
    try {
      if (t.cdr() == null) {
        List q = (List) bfsQ.firstElement();
        bfsQ.removeElement(q);
        l = bfs(q, key);
      }
      else
        l = bfs(t.cdr(), key);
    } catch (NullPointerException e) {
       e.printStackTrace();
    }
      
    return l;
  }
      
}
      
        

    
