/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Four11;
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * The Four11 Server module. The server is started on the server machine
 * and it listens in on the port given in the configuration file 
 * "Four11.conf" for new connections. Clients are expected 
 * to use the Four11Stub module to connect to the server. The Four11Stub 
 * module sets up the socket streams and provides primitives to send and
 * receive database information in the form of strings. The client is 
 * expected to parse the reply to obtain the necessary information
 *
 * An example of using the service is given below.<br>
 * <tt><br>
 *       ...<br>
 *       Four11Stub netInfo = new Four11Stub();<br>
 *       String nbors = netInfo.send("(net: KUNet (node: wigner nbors: *)");<br>
 *          // the '*' indicates the information sought. Please make sure<br>
 *          // that the property name ends with ':' and is separated from<br>
 *          // the property value by a whitespace (i.e. a space, newline or<br>
 *          // tab).<br>
 * <br>
 *        // to disconnect use <br>
 *       netInfo.send("(disconnect)"); <br>
 * </tt>
 *
 * @see Four11Stub
 */


public class Four11 {
  private static int DEFAULT_PORT = 9411;
  private static int connectionID = 1;

  /**
   * The Four11 server takes the name of a network configuration file as an argument.
   * On startup the Four11 server reads a Four11 setup configuration file to 
   * determine the port on which active nodes will attempt to connect to it. 
   * The setup configuration file is named <tt>Four11.conf</tt> and must be
   * identical at all active nodes and the Four11 server.
   * The Four11 server then creates a in-memory copy of the network topology
   * using the information from the network configuration file. The server then
   * waits for active nodes to connect. When an active node makes a request,
   * a separate thread is started that handles its query.
   *
   * @see MessageHandler
   */

  public static void main(String argv[]) {

    String network = argv[0];
    /**		
    DBReader reader = null;
    try {
      // start a thread which reads the database periodically
      reader = new DBReader(argv[0]);
      Thread readThread = new Thread(reader);
      readThread.setDaemon(true);
      readThread.start();
    } catch (Exception e) {
        System.err.println("Four11: Could not read database for network " + argv[0]);
        Thread.currentThread().stop();
    }
    ***/

    try {
        // open file Four11.conf for reading 
	BufferedReader fis = new BufferedReader(new FileReader("Four11.conf"));

        // read a line from the file and parse it to get host and port
        
        String line = null;
        String server = null;
        int listenPort = DEFAULT_PORT;
        while ((line = fis.readLine()) != null) {
          StringTokenizer st = new StringTokenizer(line, " \t\n\r");

          // file is of the format host: hostname port: portnumber
          
          while (st.countTokens() > 0) {
            if (st.nextToken().equals("host:")) {
              server = st.nextToken();
            }
            else {
              listenPort = Integer.decode(new String(st.nextToken())).intValue();
            }
	  }
	}
        //create new server socket on port 
        ServerSocket serverSoc = new ServerSocket(listenPort);
			

        // loop on accept for new connections
        while (true) {
          Socket newSoc = serverSoc.accept();
          // let new thread handle queries from node
          MessageHandler mh = new MessageHandler(newSoc, network, connectionID);
          connectionID++;
          new Thread(mh).start();
        }
      } catch (Throwable t) {
        t.printStackTrace();
      }
  }
}
