/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

package magician.Four11;
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;



/** 
 * The Four11Stub class is used by a client wishing to obtain information
 * from the Four11 server. See the description of the <tt>Four11</tt> class 
 * for an example of its use.
 *
 * @see Four11
 */

public class Four11Stub {
  private static int DEFAULT_PORT = 9411;
  private DataInputStream dis;  // input stream to read from socket
  private DataOutputStream dos; // output stream to write to socket
  private String serverHost;    // name of the machine hosting the Four11 server
  private int port = DEFAULT_PORT; // port the Four11 server listens to, default 9000.
  private Socket  clientSoc = null;

/**
 * Default constructor reads configuration file "Four11.conf" to obtain
 * name of the machine hosting the Four11 server and the port it listens to.
 */

  public Four11Stub() {
   try {
    // open file Four11.conf for reading 
     FileReader fr = new FileReader("../Four11/Four11.conf");
     BufferedReader fis = new BufferedReader(fr);

    // read a line from the file and parse it to get host and port
    String line = null;
    while ((line = fis.readLine()) != null) {
      StringTokenizer st = new StringTokenizer(line, " \t\n\r");

      // file is of the format host: hostname port: portnumber

      while(st.countTokens() > 0) {
        if (st.nextToken().equals("host:")) {
          serverHost = st.nextToken();
        }
        else {
          port = Integer.decode(st.nextToken()).intValue();
        }
      }
    }
    try {
      fis.close();
      fr.close();
    } catch (Exception r) {
      r.printStackTrace();
    }

    // open socket to server
    clientSoc = new Socket(serverHost, port);
    
    // open data streams to remote socket
    dis = new DataInputStream(clientSoc.getInputStream());
	dos = new DataOutputStream(clientSoc.getOutputStream());
   } catch (Throwable t) {
	t.printStackTrace();
   }
  }

/**
 * Use this constructor to use a Four11 server not written in the configuration
 * file.
 * @param newServer     a String containing the fully qualified name of the host 
 * @param newPort        the new port to be used
 */

  public Four11Stub(String newServer, int newPort) {
   try {
    // open socket to new server
    clientSoc = new Socket(newServer, newPort);
    
    // open data streams to remote socket
    dis = new DataInputStream(clientSoc.getInputStream());
	dos = new DataOutputStream(clientSoc.getOutputStream());
   } catch (Throwable t) {
    t.printStackTrace();
   }
  }

 /**
   * returns connection identifier for active node
   * @return the connection identifier
   */
  public int getID () {
    int ID = 0;
    try {
      ID = dis.readInt ();
    } catch (IOException e) {
      e.printStackTrace();
    }
    return ID;
  }

/**
 * sends a message to the Four11 server and returns the reply in the form of
 * a string. This interface is to be used if a reply is not expected for the
 * message. Usually used to send a "disconnect" message to the server.
 * 
 * @param message    the request in the form of a String
 * @param needReply  set to true if reply is expected for sent message  
 * @return  the reply in the form of a String
 */
  private String send(String message, boolean needReply) {
   String reply = null;
   try {
    dos.writeUTF(message);
    dos.flush ();
    if (needReply) {
      reply = dis.readUTF();
      if (reply.equals("()")) {
        reply = null;
      }
    }
   } catch (EOFException eoe) {
   } catch (SocketException se) {
   } catch (Exception e) {
      e.printStackTrace();
   }
   return reply;
  }

   /**
     * sends the request to the Four11 server. The client is expected to
     * wait for the reply.
     * @param message the request
     * @return the reply string
     */
  public String send(String message) {
    return send(message, true);
  }

  /**
     * sends a message to the Four11 server. No reply is returned and the
     * client is not expected to be waiting.
     * @param message to send
     */
  public synchronized void sendBye(String message) {
    send(message, false);
    try {
      dis.close();
      dos.close();
      clientSoc.close();
    } catch (EOFException eo) {
    } catch (SocketException sce) {
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

}
