/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Four11;
import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import magician.PerfTools.Sleeper;

 /**
 * The MessageHandler is started by the Four11 server when a new request arrives
 * from an active node. The Four11 server starts a thread that
 * handles the request and returns the reply. The request is actually sent
 * to the <tt>AssocList</tt> object for resolving. The reply returned by
 * the <tt>AssocList</tt> object is sent back to the requestor.
 *
 * @see Four11
 * @see AssocList
 */
public class MessageHandler implements Sleeper, Runnable {
  private static int TIMEOUT_PERIOD = 30 * 60 * 1000; // milliseconds
  private final static int ST_IDLE = 0;
  private final static int ST_WAIT = 1;
  private final static int ST_CLEANUP = 2;
  private final static int ST_DISCONNECT = 3;
  private int state;
  private Socket soc;
  private DataInputStream bis;
  private DataOutputStream bos;
  private AssocList netDB;
  private String netname;
  
  /**
   * initializes handler with network name and socket to listen on
   * @param soc the socket on which requests are made
   * @param network the name of network
   * @param connID the connection identifier for the active node
   */ 
    public MessageHandler (Socket soc, String network, int connID) {
    try {
      this.soc = soc;
      netname = network;
      bis = new DataInputStream(soc.getInputStream());
      bos = new DataOutputStream(soc.getOutputStream());
      // send connection ID to calling node
      bos.writeInt(connID);
      bos.flush();
      netDB = new AssocList(network);
      state = ST_IDLE;

      // Alarm alrm = new Alarm(this, TIMEOUT_PERIOD);
      // alrm.start();
    } catch (Throwable e) {
       // e.printStackTrace();
    }
  }

  /**
   * called by timer thread to periodically read the configuration file and
   * reconstruct the in-memory network configuration.
   */
  public void wakeup() {
    netDB = new AssocList(netname);
  }
 
   /**
    * loops forever waiting for requests and sends them to <tt>AssocList</tt> for
    * processing
    * @see AssocList
    */
  public void run() {
    while (state != ST_DISCONNECT) {
      try {  
        switch (state) {
          case ST_IDLE:
            String request = bis.readUTF();

            if (request.equals("(disconnect)")) {
              // System.out.println("disconnecting");
              state = ST_DISCONNECT;
              break;
            }
                     
            // else pass request to database
            String reply = netDB.get(request);
            
            // send reply back to requestor
            
            bos.writeUTF(reply);
	    bos.flush ();

            break;
                  
          case ST_CLEANUP:
            break;
          
        }
      } catch (EOFException e) {
        // pass on interrupt to DBReader
         // e.printStackTrace();
        Thread.currentThread().stop();
	/**
        dbreader.wakeup();
        // re-establish database
        netDB = dbreader.getDB(); // get database from DBReader
	**/
      } catch (IOException e) {
         Thread.currentThread().stop();
      } catch (Throwable t) {
           // t.printStackTrace();
          Thread.currentThread().stop(); 
      }  
    }
    try {
      Thread.currentThread().sleep(100);
    } catch (InterruptedException ie) {
      ie.printStackTrace();
    }
    try {
      bis.close();
      bos.close();
      soc.close();
    } catch (Exception se) {
      se.printStackTrace();
    }
  }
}
