/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

package magician.Four11;
import java.lang.*;
import java.io.*;
 //import magician.Four11.Token;

/**
 * The Tokenizer class parses an input stream and derives tokens from the stream.
 * This class is used by the Four11 server to read the configuration file and create
 * an in-memory copy of the configuration.
 * The next token in the stream is obtained by making a call to <tt>nextToken<tt>.
 * Tokens can be of type <tt>TOK_WORD, TOK_LT_PAREN, TOK_RT_PAREN or TOK_EOM</tt>.
 * If the tokenizer is to be used to read from a file, first use <tt>setCharAsByte(true)</tt> so that the
 * file stream is read properly.
 *
 * @see Four11
 * @see Token
 */
public class Tokenizer {

        /**
          * The stream used to read tokens from the configuration file
          */
	protected PushbackInputStream is;

         /**
          * The last <tt>Token<tt> read from the stream
          */
	protected Token tok;

	private char ch;             // next char in stream
	private static final int TOK_EOM = 0x0;
	private static final int TOK_LT_PAREN = 0x1;
	private static final int TOK_RT_PAREN = 0x2;
	private static final int TOK_WORD = 0x3;
	private boolean useByteChar; // treat char as single byte
                                 // needed to read in ASCII files
	
/**
 * Constructor requires an Inputstream to read the tokens from.
 * @param iis    a stream of type @see java.io.InputStream
 */
	public Tokenizer(InputStream iis) {
		this.is = new PushbackInputStream(iis, 4);
		tok = null;
		useByteChar = false;  // default char is 2 bytes
	}

/**
 * set this to true if reading from a filestream.
 * @param  opt   boolean value indicating if <tt>char</tt> is to be read in as a <tt>byte</tt>.
 */
	public void setCharAsByte(boolean opt) {
		useByteChar = opt;
	}
	
	private void readNextChar() {
		try {
			int ch1 = is.read();
			if (useByteChar) {
				ch = (char) ((0 << 8) + (ch1 << 0));	
			}
			else {
				int ch2 = is.read();
				ch = (char) ((ch1 << 8) + (ch2 << 0));
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void unreadChar() {
		try {
			
			is.unread((ch >>> 0) & 0xFF);
			if (!useByteChar)
				is.unread((ch >>> 8) & 0xFF);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private boolean whiteSpace() {
		if (ch == ' ' || ch == '\t' || ch == '\n')
			return true;
		else
			return false;
	}
	
	private boolean terminator() {
		if (whiteSpace() || ch == '(' || ch == ')' || ch == ';')
			return true;
		else
			return false;
	}
		
	private void skipWhiteSpace() {
		do {
			readNextChar();
		} while (whiteSpace());
	}
	
	private String readString() {	
		int i = 0;
		char buf [] = new char [20];

		while (!terminator()) {
			buf[i] = ch;
			if (i >= buf.length - 1) {
				char newBuf [] = new char[buf.length * 2];
				System.arraycopy(buf, 0, newBuf, 0, buf.length);
				buf = newBuf;
			}
			readNextChar(); i++;
		}
			
		unreadChar();
	
		return new String(buf, 0, i);
	}

/**
 * returns the next token read from the input stream as an object of type
 * <tt>Token</tt>. The type and value of the token can be obtained by using 
 * the methods of the <tt>Token</tt> class.
 */
	public Token nextToken() {
		tok = new Token();
		skipWhiteSpace();
		
		switch (ch) {
			case '(' : 
				tok.setType(TOK_LT_PAREN);
				tok.setValue(new String("("));
				break;
			case ')' : 
				tok.setType(TOK_RT_PAREN);
				tok.setValue(new String(")"));
				break;
			case ';':
				tok.setType(TOK_EOM);
				tok.setValue(new String("EOM"));
				break;
			default: 
				String value = readString();
				tok.setType(TOK_WORD);
				tok.setValue(value);
		}
		
		return tok;
	}
}
