/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/


import java.util.*;
import java.lang.*;
import java.io.*;
import magician.Node.*;

/**
 * Sends SmartPackets into the Active Network.
 * Presents a prompt to the user. User sends a SmartPacket by
 * typing "send <i>classname</i>" at the prompt.
 * Session terminates when user types "quit"
 */
class ActiveHostTPut {

  public static void main(String args[]) {
    // Check to make sure the local virtual host name was specified
    if ((args.length != 2) && (args.length != 4)) {
      System.err.println("Usage: java ActiveHostTPut <local_host_name> " + 
                         "<local_net_name>" +
			 "-trace [<trace_output_stream>]");
      System.err.println("     where <trace_output_stream> is one of ");
      System.err.println("           -      : to post events to console");
      System.err.println("         <file>   : to post events to named file");
      System.err.println("         host:port: to post events to named host on named port");

      System.exit(-1);
    }
    // Gather arguments
    String ourHostId = args[0];
    String ourNetId = args[1];
    String traceOutputName = null;
    int iterations = 1;
    int sleepintvl = 5;
    int numargs = 2;
    while (numargs < args.length) {
      if (args[numargs].equals("-trace")) {
        traceOutputName = args[++numargs];
        numargs++;
      } 
    }

    ActiveNode ANode = new ActiveNode (ourHostId, ourNetId, traceOutputName);
    new ActiveHostTPut ();
  }

  /**
   *  Constructor
   */
  public ActiveHostTPut () {
    /** 
    // Get list of portnames
    UnsynchVector PortNames = ActiveNode.getNeighbors ();

    // Send out on the first port available 
    String port = (String) PortNames.elementAt (0);
    **/

    // Wait for routing to stablize
    try {
      Thread.sleep (500);
    } catch (InterruptedException e) {
    }
    
    boolean nextCmd = true;
    String cmdLine = null;

    BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
   
    int pktid = 1;
    
    Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
    // sit in loop and wait for user input
    String classname = null;
    while(nextCmd) {
      try {
        System.out.print("Command> ");
        cmdLine = in.readLine();
        StringTokenizer st = new StringTokenizer(cmdLine, " \t\n\r");
	int nArgs = st.countTokens();
        if (nArgs == 0)
          continue;
	else if (nArgs == 2 || nArgs > 4) 
	  throw new IllegalArgumentException();

        String cmd = st.nextToken();
        if (cmd.startsWith("q")) {
           nextCmd = false; 
           System.exit(0);
        } else if (! cmd.startsWith("s")) {
           System.err.println("  Unknown command, use <s>end or <q>uit");
           continue;
        }

        classname = st.nextToken();
        String destHost = st.nextToken();
        int iterations = 1;
        int sleepintvl = 5;
        if (st.hasMoreTokens()) {
          // last argument is environment id
          iterations = Integer.decode(st.nextToken()).intValue();
	      }
        if (st.hasMoreTokens()) {
          sleepintvl = Integer.decode(st.nextToken()).intValue();
	      }

        // send this using KU Env id
        for (int iter = 0; iter < iterations; iter++) {
          // System.err.println("ActiveHostTPut: send packet of type " + classname);
          Object smartpkt = 
             (Object) ((Class.forName(classname)).newInstance());
          // System.err.println("ActiveHostTPut: created object of type " + smartpkt.getClass().getName());

          ActiveNode.SendSmartPacket (11, destHost, smartpkt);
          // System.err.println("ActiveHostTPut: send packet " + pktid);
          pktid++;
          Thread.currentThread().sleep(sleepintvl);
	}

        try {
          Thread.currentThread().sleep(500);
        } catch (InterruptedException e) {
        }

      } catch (ClassNotFoundException cfne) {
        // class named on command line not in classpath
        System.err.println(classname + ".class not found. Please make sure that the class is in your classpath.");
      } catch (IllegalArgumentException ice) {
        // incorrect number of arguments
        System.err.println("Incorrect number of arguments in command.");
        System.err.println("Usage: (s)end <packet class> <destination>");

      } catch (Exception e) {
	//e.printStackTrace();
      }
    }
  }
}
