/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Node;

import java.lang.*;
import java.io.*;

/**
 * Defines constants  specified in the RFC for Active Nets Encapsulation Protocol
 * 
 */

public interface ActiveNetsConstants {
  /**
   * source identifier
   */
  final static int TLV_SRCIDENT  = 1;
  /**
   * destination identifier
   */
  final static int TLV_DESTIDENT = 2; 

  /**
   * integrity checksum
   */
  final static int TLV_INTEGRITY = 3;

  /**
   * network-network authorization scheme/paramters
   */
  final static int TLV_AUTH      = 4;

  /**
   * Reference to Class definition
   */
  final static int TLV_CLASSDEF  = 5; 

  /**
   * Reference to serialized object
   */
  final static int TLV_OBJECTDEF = 6;

  /**
   * Reference to the typeid carried in the payload
   */
  final static int TLV_TYPEDEF   = 7;

  /**
   * the length of the scheme identifier
   */
 final static int TLV_SCHEMEIDLENGTH = 32;

  /** 
   * Addressing scheme identifiers
   */
  final static int TLV_IPv4ADDRESS = 1;
  final static int TLV_IPv6ADDRESS = 2;
  final static int TLV_802_3ADDRESS = 3;

  /**
   * Environment Ids 
   */
 final static int KU_TYPEID = 11;
 final static int MIT_TYPEID = 18;

  /**
   * Type id creation related constants
   */
  
  /**
   * identifier for SHA hash algorithm
   */
  final static int SHA = 0;

  /**
   * identifier for MD5 hash algorithm
   */
  final static int MD5 = 1;

  /**
   * list of string identifiers for hash algorithms
   */
  final static String[] HASHNAME = {"SHA", "MD5"};

  /**
   * list of hash-lengths for hash algorithms
   */
  final static int[] HASHLENGTH = {20, 16};

  /**
   * hash algorithm in use
   * change this when new algorithm is supposed to be the default
   * current default is "MD5"
   */
  static int CURRENT_HASH = MD5;

  /**
   * hash name in use
   */
  final static String HASHALG = HASHNAME[CURRENT_HASH];

  /**
   * length of type identifier, based on current hash algorithm
   */
  final static int TYPEIDLENGTH = HASHLENGTH[CURRENT_HASH];

}
