/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Node;

import java.lang.*;
import java.util.*;
import java.net.*;
import magician.Utils.*;
import magician.PerfTools.*;
import magician.Four11.*;

/**
 * The main entry point for starting up an Active Node. It contacts the Four11 Information server
 * to obtain configuration information, starts up the PortManager and the Demultiplexer so that
 * they can send and receive Active Network Encapsulation Protocol (ANEP) packets. It also
 * starts up all the processing environments available at the Active Node.
 * 
 * @see PortManager
 * @see ActiveNodeManager
 * @see Demultiplexer
 */

public class ActiveNode implements ActiveNetsConstants {
  private static String ourNodeName;
  private static String ourNetName;
  private static Hashtable PortManagerTable;
  private static UnsynchVector PortManagers;
  Four11Stub Four11Server;
  int Four11ConnID;
  public static Hashtable ProcEnvs = null;
  static Hashtable NborHostTable;
  static Hashtable NborPortTable;
  static EventManager EventMgr;

  /**
   * gives the node names of the neighbors of this Active Node
   * @return a Vector consisting of the names of the nodes
   */
  public static UnsynchVector getNeighbors() {
    UnsynchVector Nbors = new UnsynchVector();
    for (Enumeration e = NborHostTable.keys(); e.hasMoreElements(); ) {
      Nbors.addElement((String) e.nextElement());
    }
    return Nbors;
  }

  /**
   * returns the name of the machine on which a virtual Active Node resides
   * @param nodeName the name of the Active Node
   * @return a String object containing the name of the physical host
   */
  public static String getPhysicalHostName(String nodeName) {
    return (String) NborHostTable.get(nodeName);
  }

  /**
   * get the PortManager for a specified UDP port
   * @param udpPort the UDP port 
   * @return the PortManager object associated with the port
   */
  public static PortManager getPortManagerForPort(int udpPort) {
    return (PortManager) PortManagerTable.get(String.valueOf(udpPort));
  }

  /**
   * get all PortManager for this node
   * @return a Vector containing the PortManager objects 
   */
  public static UnsynchVector getPortManagers() {
    return PortManagers;
  }

  /**
   * get the UDP ports a node is listening to
   * @param nodeName the name of the Active Node
   * @return a space separated string containing the port numbers
   */
  public static String getPortsForNode(String nodeName) {
    return (String) NborPortTable.get(nodeName);
  }

  /**
   * Get this node's virtual name
   * @return the node name as a String
   */
  public static String GetNodeName () {
    return ourNodeName;
  }

  /**
   * Get this node's virtual net name
   * @return the net name as a String
   */
  public static String GetNetName () {
    return ourNetName;
  }
  
  /**
    * returns the event manager initialized for this node
    * @return the event manager object
    */
  public static EventManager getEventManager() {
    return EventMgr;
  }

  /** 
   * interface for Node Manager to send packets to the correct port manager
   * @param node the name of the destination to send packet to
   * @param port the port on which to send Smartpacket
   * @param packetBytes the bytes comprising the Smartpacket
   * @return true if the packet was sent successfully, false otherwise
   *
   * @see ExecEnvironment
   * @see ActiveNode
   * @see PortInterface
   */
  public synchronized static boolean send(String node, int port, byte[] packetBytes) {
    System.err.println("ActiveNode (send): not implemented");
    return false;
  }

  /**
   * temporary interface to send Smartpackets from host
   * @param EnvID the processing environment to use to process 
   * @param destination the node to send the Smartpacket to
   * @param SmartPkt the Smartpacket object
   */
  public static void SendSmartPacket(int EnvID, String destination, Object SmartPkt) {
    try {
      // simply call the Active Node Manager's send method with the above arguments
      ExecEnvironment ExecEnv = (ExecEnvironment) ProcEnvs.get(new Integer(EnvID));
      ExecEnv.Process(destination, SmartPkt);
    } catch (NullPointerException e) {
	// Environment of that id not found here
        // what to do here?
    }
  }  

/*************
  static {
    try {
      System.loadLibrary("TagMagician");
      System.out.println("TagMagician loaded");
    } catch (Error e) {
      System.out.println("problem with library loading : " + e);
    }
  }
************/

  /**
   * initializes the Active Node
   * @param thisHost name of this active node
   * @param thisNet name of the network
   * @param traceStream name of logging stream
   */
  public ActiveNode(String thisHost, String thisNet, String traceStream) {
    //cpu project


    ourNodeName = thisHost;
    ourNetName = thisNet;

    ProcEnvs = new Hashtable();

    // Information Server socket and data streams
    Socket InformationServer = null;

    // Connect to information server
    Four11Server = new Four11Stub ();
  
    // get connections from Information Server and set them up
    try {
      setupConnections ();
    } catch (UnknownHostException e) {
      System.err.println("Host mismatch exception from Four11 Server: " + e.getMessage());
      Four11Server.sendBye("(disconnect)");
      System.exit(1);
    }

    Four11Server.sendBye("(disconnect)");
    // put the processing environments at the Active Node in a table indexed by 
    ActiveNodeManager env = new ActiveNodeManager (thisHost, thisNet, traceStream);
    ProcEnvs.put(new Integer(KU_TYPEID), env);
    // and so on.. for MIT, Penn, etc

    Demultiplexer Demux = new Demultiplexer(ProcEnvs);
    EventMgr = Demux.getEventManager();
    // start the demux
    Thread dmux = new Thread(Demux);
    dmux.setPriority(Thread.MAX_PRIORITY);
    dmux.start();

    for (Enumeration e = ProcEnvs.elements(); e.hasMoreElements(); ) {
      Thread envMgr = new Thread((ExecEnvironment) e.nextElement());
      envMgr.setPriority(Thread.MAX_PRIORITY);
      envMgr.start();
    }
    
    Thread.currentThread().yield();
  }

  /**
   * converts an int into a Type field
   * @param typeInt  an int repsenting the type 
   * @return a two byte field representing the type
   */
  public byte[] setType(int typeInt) {
    byte[] type = new byte[2];
    type[0] = (byte) ((typeInt >> 8) & 0xFF);
    type[1] = (byte) ((typeInt >> 0) & 0xFF);
    return type;
  }

  /**
   * Get the connections from the information server
   * @exception UnknownHostException if active node is run on different host than one set up in information server
   */
  private void setupConnections () throws UnknownHostException {
    PortManagerTable = new Hashtable();
    NborHostTable = new Hashtable();
    NborPortTable = new Hashtable();
    PortManagers = new UnsynchVector();

    Four11ConnID = Four11Server.getID ();

    // get this node's physical host
    String Query = "(net: " + ourNetName + " (node: " + ourNodeName + " host: *))";
    String Reply = Four11Server.send (Query);

    // check to see if it corresponds with the host on which active node is running
    InetAddress lclhost = InetAddress.getLocalHost();
    InetAddress anHost = InetAddress.getByName(Reply);
    if (! lclhost.getHostAddress().equals(anHost.getHostAddress())) {
      throw new UnknownHostException("Active Node is supposed to be running on " + anHost.getHostName() + " instead of " + lclhost.getHostName());
    }

    // get this node's physical host
    // Query = "(net: " + ourNetName + " (node: " + ourNodeName + " host: *))";
    // Reply = Four11Server.send (Query);
    // put this as an entry in the hosttable
    NborHostTable.put(ourNodeName, Reply);
    // get this node's neighbors
    Query = "(net: " + ourNetName + " (node: " + ourNodeName + " nbors: *))";

    Reply = Four11Server.send (Query);
    // Tokenizer to parse returned list
    StringTokenizer st = new StringTokenizer(Reply, " ()\n\t;");

    // Other host's name, port number and whether we connect to them
    String NeighborName = null, physHost = null;

    // Loop, reading neighbor names from information server
    String Ports = new String(" ");
    while (st.hasMoreTokens()) {
      // read in the virtual names of the neighbors and the physical machines on which they reside
      NeighborName = st.nextToken();
      // get this node's physical host
      Query = "(net: " + ourNetName + " (node: " + ourNodeName + " host: *))";

      Reply = Four11Server.send (Query);

      // put this as an entry in the hosttable
      NborHostTable.put(NeighborName, Reply);

      // read in and discard interface address
      st.nextToken();

      // read in port number       
      String PortNumber = st.nextToken();
      Ports = Ports.concat(PortNumber + " ");

      NborPortTable.put(NeighborName, PortNumber);
      // discard link bw 
      st.nextToken();
    }


    // Loop, reading port numbers from information server
    st = new StringTokenizer(Ports, " \n\t");
    while (st.hasMoreTokens()) {
      String listenPort = st.nextToken();

      try {
        // Create new port manager thread and start it
        PortManager PortMgr = new PortManager(Integer.decode(listenPort).intValue());
        PortManagers.addElement(PortMgr);
        PortManagerTable.put (listenPort, PortMgr);
        Thread pMgr = new Thread(PortMgr);
        pMgr.setPriority(Thread.MAX_PRIORITY);
        pMgr.start();
      } catch (SocketException e) {
         e.printStackTrace();
      }
    }
  }
}
