/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Node;

import java.lang.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.security.*;
import magician.PerfTools.*;
import magician.Utils.*;

/**
 * The Active Node Manager is the top-level manager of the Active Node. It starts up other entities of the 
 * Active Node such as the Resource Manager, Routing Manager and the Event 
 * Manager. It also provides some primitives for the user Smartpacket to 
 * create and manage small state.
 * 
 * @see ResourceMgr
 * @see RoutingMgr
 * @see magician.PerfTools.EventManager
 */

public class ActiveNodeManager implements ExecEnvironment, PerfConstants  {

  // Other Managers
  private DataOutputStream ps = null;
  private static String Services = "";
  private static Hashtable Servers = new Hashtable ();

  private static Queue ExecPktsQ = new Queue(50);

  /** 
   * temporary hack while the SS Manager is cleaned up
   */
  private static Hashtable SmallState = new Hashtable();

  private static RoutingMgr RouteMgr = null;
  // Resource Manager
  private static ResourceMgr ResMgr = null;

  private static EventManager EventMgr = null;
  private NetLoggerEventManager NLMgr = null;

  // Small State manager
  // CreditManager creditMgr;
  public static PortInterface transmitter; /* SB changed to public */

  private static PacketAssemble PktAssem;

  // Name of this node and net
  private static String ourNodeName;


  private static String ourNetName;
 
  // Output Stream for Traces
  private static OutputStream traceOutputStream = null;

  /**
   * returns the object stored in the small state associated with given key.
   * If the contents of the small state are being accessed by another SmartPacket,
   * then a NoSuchElementException exception is thrown. The normal procedure is
   * to then yield the CPU and try to reacquire the small state.
   *
   * @param key a string identifying the small state
   * @return contents of the small state
   * @exception StateNotFoundException if the small state associated with the key does not exist
   * @exception StateEmptyException if small state exists but is empty, which is the case when some other SmartPacket is accessing the contents. 
   */
  public static Object GetSmallState(String key) 
     throws StateNotFoundException, StateEmptyException {
    // add typeid of calling smartpacket to ensure uniqueness
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    try {
    thr.setPriority(prio + 1);
    } catch (IllegalArgumentException iae) {
      System.err.println("ActiveNodeManager: cannot set prio to " + (prio+1));
    }
    KUSmartPacketV2 sp = SPThread.currentSPThread().getSmartPacketforSPThread();
    byte[] type = sp.getTypeId();
    String _key = key + new String(type); 
    if (! SmallState.containsKey(_key)) {
      thr.setPriority(prio);
      throw new StateNotFoundException();
    }
    Queue state = (Queue) SmallState.get(_key);
    Object stateObj = null;
    // synchronized (state) {
      try {
        stateObj = state.remove();
      } catch (InterruptedException nsee) {
        thr.setPriority(prio);
        throw new StateEmptyException();
      }
      // }
    thr.setPriority(prio);
    return stateObj;
  }

  /**
   * returns the object stored in the small state associated with given key.
   * If the contents of the small state are being accessed by another SmartPacket,
   * then a NoSuchElementException exception is thrown. The normal procedure is
   * to then yield the CPU and try to reacquire the small state.
   *
   * @param key a string identifying the small state
   * @param timeout time period to wait for, else throw StateEmptyException
   * @return contents of the small state
   * @exception StateNotFoundException if the small state associated with the key does not exist
   * @exception StateEmptyException if small state exists but is empty, which is the case when some other SmartPacket is accessing the contents. 
   */
  public static Object GetSmallState(String key, long timeout)
     throws StateNotFoundException, StateEmptyException {
    // add typeid of calling smartpacket to ensure uniqueness
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    try {
    thr.setPriority(prio + 1);
    } catch (IllegalArgumentException iae) {
      System.err.println("ActiveNodeManager: cannot set prio to " + (prio+1));
    }
    KUSmartPacketV2 sp = SPThread.currentSPThread().getSmartPacketforSPThread();
    byte[] type = sp.getTypeId();
    String _key = key + new String(type); 
    if (! SmallState.containsKey(_key)) {
      thr.setPriority(prio);
      throw new StateNotFoundException();
    }
    Queue state = (Queue) SmallState.get(_key);
    Object stateObj = null;
    // synchronized (state) {
      try {
        stateObj = state.remove(timeout);
        if (stateObj == null) {
          thr.setPriority(prio);
          throw new StateEmptyException();
        }
      } catch (InterruptedException nsee) {
        thr.setPriority(prio);
        throw new StateEmptyException();
      }
      // }
    thr.setPriority(prio);
    return stateObj;
  }

  /**
    * sets the contents of the small state
    * @param key a string identifying the small state
    * @param stateObj the content to be deposited
    * @exception StateNotFoundException if state named by key does not exist
    */
  public static void SetSmallState(String key, Object stateObj) throws StateNotFoundException {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    try {
    thr.setPriority(prio + 1);
    } catch (IllegalArgumentException iae) {
      System.err.println("ActiveNodeManager: cannot set prio to " + (prio+1));
    }     

    // add typeid of calling smartpacket to ensure uniqueness
    KUSmartPacketV2 sp = SPThread.currentSPThread().getSmartPacketforSPThread();
    byte[] type = sp.getTypeId();
    String _key = key + new String(type); 
    if (! SmallState.containsKey(_key)) {
      thr.setPriority(prio);
      throw new StateNotFoundException();
    }
    Queue state = (Queue) SmallState.get(_key);
    // synchronized(state) {
      state.insert(stateObj);
      // }

    thr.setPriority(prio);
    return;
  }

  /**
    * creates small state. It is the task of the user to check if small state 
    * same name already exists. The small state created by this method is accessible
    * only to SmartPackets of the same type as the one that created it. To share 
    * state, use <tt>CreateGlobalState()</tt> method.
    * @param key a string identifying the small state
    * @exception StateExistsException if state named by given key exists
    */
  public synchronized static void CreateSmallState(String key) throws StateExistsException {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio + 1);
    // add typeid of calling smartpacket to ensure uniqueness
    KUSmartPacketV2 sp = SPThread.currentSPThread().getSmartPacketforSPThread();
    byte[] type = sp.getTypeId();
    String _key = key + new String(type); 
    if (SmallState.containsKey(_key)) {
      thr.setPriority(prio);
      throw new StateExistsException();
    }
    SmallState.put(_key, new Queue());
    thr.setPriority(prio);
    return;
  }

  /**
    * removes small state associated with key
    * @param key a string identifying the small state
    * @exception StateNotFoundException if named state does not exist
    */
  public static void DestroySmallState(String key) throws StateNotFoundException {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio + 1);
    // add typeid of calling smartpacket to ensure uniqueness
    KUSmartPacketV2 sp = SPThread.currentSPThread().getSmartPacketforSPThread();
    byte[] type = sp.getTypeId();
    String _key = key + new String(type); 
    if (! SmallState.containsKey(_key)) {
      thr.setPriority(prio);
      throw new StateNotFoundException();
    }
    SmallState.remove(_key);
    thr.setPriority(prio);
    return;
  }


  /**
   * returns the object stored in the global state associated with given key.
   * If the contents of the global state are being accessed by another SmartPacket,
   * then a NoSuchElementException exception is thrown. The normal procedure is
   * to then yield the CPU and try to reacquire the global state.
   *
   * @param key a string identifying the global state
   * @return the contents of the global state
   * @exception NullPointerException if the global state associated with the key does not exist
   * @exception NOSuchElementException if global state exists but is empty, which is the case when some other SmartPacket is accessing the contents. 
   */
  public static Object GetGlobalState(String key) 
     throws NullPointerException, NoSuchElementException {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio + 1);
    Queue state = (Queue) SmallState.get(key);
    Object stateObj = null;

    // synchronized (state) {
      try {
       stateObj = state.remove(); // SB removed timeout I had added
      } catch (InterruptedException nsee) {
        thr.setPriority(prio);
        throw new NoSuchElementException();
      } catch (NullPointerException npe) {
        thr.setPriority(prio);
        throw new NullPointerException();
      }
      // }
    thr.setPriority(prio);
    return stateObj;
  }

  /**
    * sets the contents of the global state
    * @param key a string identifying the global state
    * @param stateObj the content to be deposited
    * @exception StateNotFoundException if the global state associated with the key does not exist
    */
  public static void SetGlobalState(String key, Object stateObj) {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio + 1);
    // add typeid of calling smartpacket to ensure uniqueness
    Queue state = (Queue) SmallState.get(key);
    // synchronized (state) {
    try {
      state.insert(stateObj);
    } catch (NullPointerException npe) {
      thr.setPriority(prio);
      throw new NullPointerException();
    }
      // }
    thr.setPriority(prio);
    return;
  }

  /**
    * creates global state. It is the task of the user to check if global state by
    * same name already exists. The state created by this method is accessible
    * to SmartPackets of all types. To create per-type small
    * state, use <tt>CreateSmallState()</tt> method.
    * @param key a string identifying the global state
    */
  public synchronized static boolean CreateGlobalState(String key) {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio + 1);
    // add typeid of calling smartpacket to ensure uniqueness
    if (SmallState.containsKey(key)) {
      thr.setPriority(prio);
      return false;
    }
    SmallState.put(key, new Queue());
    thr.setPriority(prio);
    return true;
  }

  /**
    * removes global state associated with key
    * @param key a string identifying the global state
    */
  public static void DestroyGlobalState(String key) {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio + 1);
    SmallState.remove(key);
    thr.setPriority(prio);
    return;
  }

  /**
   * returns a list of neighbors nodes
   * @returns a Vector of adjoining neighbor nodes
   */
  public static UnsynchVector getNeighbors() {
    return ActiveNode.getNeighbors();
  }

  /**
   * Register a new service with the ActiveNodeManager. Any service can be added.
   * However, it has to be noted that the service is global in scope and can be
   * accessed by Smartpackets of other types.
   *
   * @param ServiceName the name of the service
   * @param Server the object implementing the service
   */
  public static void RegisterService (String ServiceName, Object Server) {
    Services = Services + ServiceName + " ";
    Servers.put (ServiceName, Server);
  }

 /**
  * returns the bandwidth capacity to a neighboring node. <Deprecated>
  * @param nodeName name of the neighboring node
  * @return the bandwidth in Kbps
  */
  public static int GetLinkBandwidthToNode(String nodeName) {
   
    return PortInterface.getLinkBandwidthToNode(nodeName);
  }


  /**
   * Return the list of all services
   * @return the service listing
   */
  public static String GetServices () {
    return Services;
  }

  /**
    * Check if a particular service is available
    * @param ServiceName name of the service
    * @return object implementing service, else null
    */
  public static Object HaveService (String ServiceName) {
    return Servers.get (ServiceName);
  }

 /**
  * Called by Active nodes that behave as hosts to inject smartpackets
  * <Deprecated>
  * @param Portname name of destination active node
  * @param SPObject the SmartPacket object
  * @return true if transmission was successful, false otherwise
  */
  public boolean Send(String PortName, Object SPObject) {
    return SendSmartPacket(PortName, SPObject);
  }


 /**
  * called by SmartPacket to send itself to the destination <deprecated>
  * @param Portname name of destination active node
  * @param SPObject the SmartPacket object
  * @return true if transmission was successful, false otherwise
  */
  public static boolean SendSmartPacket (String PortName, 
					           Object SPObject) {
    boolean status = transmitter.SendSmartPacket(PortInterface.getANAddress(PortName), SPObject);

    return status;
  }

  /**
   * Get this node's virtual name
   * @returns the node name as a String
   */
  public static String GetNodeName () {
    return ourNodeName;
  }

  /**
   * Get this node's virtual net name
   * @return the net name as a String
   */
  public static String GetNetName () {
    return ourNetName;
  }

  /**
   * Get this node's trace output stream
   * return an OutputStream for trace output
   * @see OutputStream
   */
  public static OutputStream GetTraceOutputStream () {
    return traceOutputStream;
  }

  /**
   * entry point for executing packets handed by the sending node <deprecated>
   * @param destination the destination of this SmartPacket
   * @param SPObject the executable object 
   */
  public static void ProcessSP(String destination, KUSmartPacketV2 SPObject) {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    prio = (prio >= Thread.MAX_PRIORITY)? Thread.MAX_PRIORITY - 1: prio;
    thr.setPriority(prio + 1);
    try {
      SPObject.Source_Address = GetNodeName();
      SPObject.Destination_Address = PortInterface.getANAddress(destination);
      PortInterface.createNewOTWPacket(SPObject);
      ExecPktsQ.insert(SPObject);
    } catch (NullPointerException e) {
       // null SPObject.
       if (SPObject == null)
         System.err.println("ActiveNodeManager: Null Smartpacket received");
       else
         System.err.println("PortInterface is null");
       e.printStackTrace();
    }
    thr.setPriority(prio);
  }
  
  /**
   * entry point for executing packets handed by the sending node <deprecated>
   * @param destination the destination of this SmartPacket
   * @param SPObject the executable object 
   */
  public void Process(String destination, Object SPObject) {
    ProcessSP(destination, (KUSmartPacketV2) SPObject);
  }

  /**
   * well-known entry method to send this environments packets for processing.
   * This method accepts the entire packet as a stream of bytes. This is useful 
   * when this VM is operating at another site <deprecated>
   *
   * @param packetbytes the stream of bytes comprising the packet
   */
  public static void ProcessSP(byte[] packetbytes) {
    if (EventMgr.traceDepthSupported(TR_STREAM)) {
      String sbuf = "PACKETSIZE=" + String.valueOf(packetbytes.length);
      EventMgr.postEvent(TR_STREAM, "ActiveNodeManager.PacketStream.Receive", System.currentTimeMillis(), 0, sbuf);
    }
    try {
      KUSmartPacketV2 SPObject = transmitter.getExecutable(packetbytes);
      packetbytes = null;
      SPObject.TypeID = PortInterface.defineType(SPObject);
      // execute(SPObject);
    } catch (NullPointerException e) {
       System.err.println("ActiveNodeManager: Null SmartPacket object");
       e.printStackTrace();
    }
  }


  /**
    * enqueues Smartpackets into ready queue
    * 
    * @param SPObject KU_SmartPacket object 
    */
  protected static void enqueue(KUSmartPacketV2 SPObject) {
    ExecPktsQ.insert(SPObject);
  }

  /**
   * entry point for executing packets received over the wire <deprecated>
   * @param packetbytes the on-the-wire representation of the SmartPacket
   */
  public void Process(byte[] packetbytes) {
    PktAssem.Process(packetbytes);
    // ProcessSP(packetbytes);
  } 


  /**
   * calls the Resource Manager and schedules packet for execution
   * @param SPObject the executable Smartpacket object
   */
  private static void execute(KUSmartPacketV2 SPObject) {
    if (EventMgr.traceDepthSupported(TR_MEASURE)) {
      String sbuf = "CLASSNAME=" + SPObject.getClass().getName(); 
      EventMgr.postEvent(TR_MEASURE, "ActiveNodeManager.Processing.Start", System.currentTimeMillis(), SPObject.PacketID, sbuf);
    }

    if (SPObject.PacketID % 2227 == 0) {	// SB
      long stTime = System.currentTimeMillis();
      System.err.print("ActiveNodeManager: init GC at time " + stTime);
      Runtime rt = Runtime.getRuntime();
      long initMem = rt.freeMemory();
      rt.gc();
      long freedBytes = rt.freeMemory() - initMem;
      long freedTime = System.currentTimeMillis() - stTime;
      System.err.println(" : freed " + freedBytes + " of memory in " + freedTime + " secs");
    }
 
    // Hand packet to Resource Manager for processing
    ResMgr.newThread(SPObject);

  }

  /** 
   * starts up the Node Manager
   */
  public void run() {
    // get the neighbors and call the PortUp method on it
    for (Enumeration e = getNeighbors().elements();
         e.hasMoreElements(); ) {
      String node = (String) e.nextElement();
      if (!node.equals(ourNodeName))
        RouteMgr.PortUp(node);
    }

    while (true) {
      try {
        KUSmartPacketV2 SPObject = (KUSmartPacketV2) ExecPktsQ.remove();
      
        execute(SPObject);
      } catch (InterruptedException ie) {
        ie.printStackTrace();
      }
    }
  }

  /**
   * Creates a SmartPacketLoader, starts all other managers, requests
   *   connection information from InformationServer, and then creates a
   *   socket and a new PortManager for each connection.
   *
   * @see magician.Four11.Four11
   */
  public ActiveNodeManager (String ourHostId, String ourNetId,
			    String traceOutputName) {
    
    ourNodeName = ourHostId;
    ourNetName = ourNetId;
    int ix = 0;
    // Set up the trace output stream
    try {
      if (traceOutputName.equals ("-")) {
	traceOutputStream = System.out;
      } else if ((ix = traceOutputName.indexOf(":")) > 0) { 
         // i.e. name is of form "xx:nn"
         // output is likely to a socket, parse name
         String trHost = traceOutputName.substring(0, ix);
         int tracePort = 0;
         try {
           tracePort = Integer.parseInt(traceOutputName.substring(ix+1, traceOutputName.length()));
           if (tracePort <= 0) throw new NumberFormatException();
         } catch (NumberFormatException nfe) {
            throw new NullPointerException("Invalid port number for socket. Setting trace output to /dev/null");
         }

         // open socket to trace host
         try {
           Socket trSoc = new Socket(trHost, tracePort);
           traceOutputStream = trSoc.getOutputStream();
         } catch (Exception se) {
            throw new NullPointerException("Cannot open connection to trace host " + trHost + " on port " + tracePort + ". Check if server is running. Setting trace output to /dev/null");
         }

      } else {
	traceOutputStream = new FileOutputStream (traceOutputName, true);
      }
    } catch (NullPointerException e) {
      System.err.println(e);
      //  traceOutputName was undefined, so set stream to /dev/null
      try {
	traceOutputStream = new FileOutputStream ("/dev/null", true);
      } catch (Exception t) {};
    } catch (IOException e) {
      //  traceOutputName caould not be opened, so set stream to /dev/null
      try {
	traceOutputStream = new FileOutputStream ("/dev/null", true);
      } catch (Exception t) {};
    }

    // Create and initialize other Managers and entities
    EventMgr = null;
    ps = new DataOutputStream(traceOutputStream);
    try {
      EventMgr = new EventManager(10);
      RegisterService("EventManager", EventMgr);

      // set up the NetLoggerEventManager as the default listener for events
      NLMgr = new NetLoggerEventManager(5000, ps);
      // let NetLoggerEventmanger listen to all events asynchronously
      EventMgr.addListenerForAllEvents((NetEventListener) NLMgr, false);
    } catch (ClassNotFoundException e) {
        e.printStackTrace();
    }

    // creditMgr = new CreditManager(getNeighbors());
    transmitter = new PortInterface();
    
    Thread transmitThr = new Thread(transmitter);
    transmitThr.setPriority(Thread.MAX_PRIORITY);
    transmitThr.start();

    //start thread to handle assmbly of packet stream received over the wire
    PktAssem = new PacketAssemble();
    Thread pktAssemThr = new Thread(PktAssem);
    pktAssemThr.setPriority(Thread.MAX_PRIORITY);
    pktAssemThr.start();

    RouteMgr = new RoutingMgr(); 
    ResMgr = new ResourceMgr(1000,10,10000);
  }
}
