/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Node;

import java.lang.*;
import java.util.*;

/**
 * A blocking queue of elements. Readers of the queue block when attempting to read when the
 * queue is empty. The readers are woken up when an element is inserted into the queue.
 */

public class HashQueue extends Hashtable {
  /**
   * constructs an empty queue
   */
  public HashQueue () {
    super();
  }
        
  /** 
   * constructs a queue of the specified size
   * @param size  the initial size of the queue
   */
  public HashQueue(int size) {
    // set the size and the capacity increment to quarter the size
    super(size, 0);
  }

  /** 
   * inserts an object into the queue. If the queue was previously empty,
   * wakes up any sleeping readers
   * @param key  the key under which data object is to be stored 
   * @param element  the data object to be placed on the queue
   */
  public synchronized Object insert(Object key, Object element) {
    boolean wasEmpty = isEmpty(); // check if queue is empty prior to inserting object
    super.put(key, element);

    if (wasEmpty) {
      // queue was empty before object was inserted in the queue, then
      // wake up any readers who are waiting
      notify();
    }           

    return key;
  } 
    
  /**
   * retrieves the element of the queue specified by given key. If the queue is empty,
   * this method blocks until one is available. 
   * @param key key for the requiree element
   * @returns the first element of the queue
   * @exception InterruptedException if operation is interrupted
   */
  public synchronized Object delete(Object key) {

    Object element = super.get(key);
    super.remove(key);
    return element;
  }
    
  /**
   * returns a reference to the element specified by given key but does not remove the element
   * from the queue. If the queue is empty, it returns null.
   * @param key key for the required element
   * @returns the element or null if there is no element associated by that key 
   */
  public synchronized Object fetch(Object key) throws InterruptedException {
    while (isEmpty()) {
      wait();
    }
      
    Object elem = super.get(key);

    // notify all waiting threads
    notify();
    return elem;
  }
}
