/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

package magician.Node;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.security.*;
import java.lang.reflect.*;
import magician.PerfTools.*;
import magician.Four11.*;

/**
 * This module accepts outgoing packets packets as byte arrays, and decides which 
 * Port must send them
 *
 * @see PortManager
 */


public class PortInterface implements Runnable, ActiveNetsConstants, PerfConstants {

  private static Hashtable NodeProperties;
  private static Vector primitiveClasses;

  private static Hashtable TypeIDTable = new Hashtable();

  private static Four11Stub Four11Server;
  private static EventManager EventMgr;
  public static long packetsSent; /* SB changed to public */
  public static long bytesSent;	/* SB added */
  private static String ourNodeName;
  private static String ourNetName;

  private static DatagramPacket DGSend;
  private static Queue  transmitQ = new Queue(100);

  /**
   * Loader for loading KU SmartPackets which use serialization
   */
  public static SmartPacketLoader SPLoader; 

  /**
   * creates a PortInterface object
   */
  public PortInterface() {
    packetsSent = 0;
    bytesSent = 0;	/* SB added */

    Four11Server = new Four11Stub();
    NodeProperties = new Hashtable();
    int id = Four11Server.getID();
    EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");
    setup();
    Four11Server.sendBye("(disconnect)");

    AckManager ackmgr = new AckManager((Hashtable) NodeProperties.get("network.nodes"));
    Thread ackThr = new Thread(ackmgr);
    ackThr.setPriority(Thread.MAX_PRIORITY);
    ackThr.start();
  }


  public void run() {
    // dequeue packets in queue and transmit them

    while (true) {
      String addr = null;
      int port = 0;
      byte[] pkt = null;
      try {
        InetPacket ipkt = (InetPacket) transmitQ.remove();
        addr = ipkt.getIPAddress();
        port = ipkt.getPortNumber();
        pkt  = ipkt.getPacketBytes();
        sendPacket(addr, port, pkt);
      } catch (Exception ioe) {
        System.err.println(ioe + ":address=" + addr + ", port = " + port);
        ioe.printStackTrace();
      }
    }
  }

  /**
   * Get the connections from the information server
   */
  private void setup() {
    SPLoader = new SmartPacketLoader();    

    ourNetName = ActiveNodeManager.GetNetName();
    // put it in properties
    NodeProperties.put("node.net", ourNetName);
 
    ourNodeName = ActiveNodeManager.GetNodeName();
    NodeProperties.put("node.name", ourNodeName);

    // get this node's descriptions
    String Query = "(net: " + ourNetName + " (node: " + ourNodeName + " IP: *))";
    String Reply = QueryFour11Server(Query);

    Hashtable thisNode = new Hashtable();
    thisNode.put("node.name", ourNodeName);
    thisNode.put("node.LLAddress", Reply);

    Query = "(net: " + ourNetName + " (node: " + ourNodeName + " host: *))";

    Reply = QueryFour11Server(Query);
    thisNode.put("node.physicalHost", Reply);

    // get the gateway for this node
    Query = "(net: " + ourNetName + " (node: " + ourNodeName + " gateway: *))";
    Reply = QueryFour11Server(Query);
    StringTokenizer st = new StringTokenizer(Reply, " ()\n\t;");
    thisNode.put("node.gateway", st.nextToken());

    thisNode.put("node.linkBandwidth" , "0");
    thisNode.put("node.neighbor", "no");


    Hashtable NetworkNodes = new Hashtable();
    NetworkNodes.put(ourNodeName, thisNode);

    // get this node's neighbors
    Query = "(net: " + ourNetName + " (node: " + ourNodeName + " nbors: *))";

    Reply = QueryFour11Server(Query);

    // Tokenizer to parse returned list
    st = new StringTokenizer(Reply, " ()\n\t;");

    // Other host's name, port number and whether we connect to them
    String NeighborName = null, physHost = null;

    // Loop, reading neighbor names from information server
    while (st.hasMoreTokens()) {
      // read in the virtual names of the neighbors and the physical machines on which they reside
      Hashtable NborProps = new Hashtable();

      NeighborName = st.nextToken();
      NborProps.put("node.name", NeighborName);

      Query = "(net: " + ourNetName + " (node: " + NeighborName + " host: *))";
      physHost = QueryFour11Server(Query);
      System.err.println("Query= " + Query + " host= " + physHost);
      NborProps.put("node.physicalHost", physHost);

      // get interface on which to contact neighbor
      String iface = st.nextToken();
      // put this as an entry in the hosttable
      NborProps.put("node.LLAddress", iface);

      String PortNumber = st.nextToken();
      NborProps.put("node.port", PortNumber);

      // get link bandwidth
      String linkBW = st.nextToken();
      NborProps.put("node.linkBandwidth", linkBW);
      NborProps.put("node.neighbor", "yes");

      NetworkNodes.put(NeighborName, NborProps);      
    }

    NodeProperties.put("network.nodes", NetworkNodes);
    // System.err.println("PortInterface: Nodes = " + NetworkNodes.toString());
  }

  /**
   * takes a string and determines if it is an IP Address of the form "%d.%d.%d.%d".
   * @param name the address to be checked
   * @return true if it is an IPAddress string, false otherwise
   */
  private static boolean isIPAddress(String name) {
    int idx = 0;
    int nextIdx = name.indexOf('.');

    int parts = 0;
    String subaddr = null;
    while (parts < 4 && nextIdx != -1) {
      subaddr = name.substring(idx, nextIdx);
      if (Integer.decode(subaddr).intValue() > 255) 
        return false;

      idx = nextIdx;
      nextIdx = name.indexOf('.', idx);
      idx++;
      parts++;
      if (nextIdx == -1)
        // possibly reached last part, read remaining string to check if it is indeed last
        nextIdx = name.length();
    }
    
    if (parts == 4 && nextIdx == -1)
      return true;
    else
      return false;
  }

  private static String QueryFour11Server(String query) {
    return Four11Server.send (query);
  }

  
  /**
   * verifies if given address is an Active Net address and
   * if not returns the AN address of the AN gateway from which
   * destination can be reached
   *
   * @param addr  the address to be checked
   * @return the corresponding Active Node name
   */
  protected synchronized static String getANAddress(String addr) {
    Hashtable nodelist = (Hashtable) NodeProperties.get("network.nodes");
    Hashtable nodeProps = null;
    String anetAddr = null;
    try {
      nodeProps = (Hashtable) nodelist.get(addr);
      anetAddr = (String) nodeProps.get("node.name");
    } catch (NullPointerException e) {
      // not in cache, get it from Four11 server
      String ourNetName = (String) NodeProperties.get("node.net");

      Four11Server = new Four11Stub();
      int id = Four11Server.getID();
      // get this node's IP address
      String Query = "(net: " + ourNetName + " (node: " + addr + " IP: *))";
      // System.err.println(Query);
      String Reply = null;
      Reply = QueryFour11Server(Query);
      if (Reply != null) {
        Hashtable newNode = new Hashtable();
        newNode.put("node.name", addr);

        newNode.put("node.LLAddress", Reply);

        Query = "(net: " + ourNetName + " (node: " + addr + " host: *))";

        Reply = QueryFour11Server(Query);
        newNode.put("node.physicalHost", Reply);

        newNode.put("node.neighbor", "no");
      
        nodelist.put(addr, newNode);
        anetAddr = addr;
      } else {
        // addr is of host outside this active network
        nodeProps = (Hashtable) nodelist.get(ourNodeName);
        anetAddr = (String) nodeProps.get("node.gateway");
        Hashtable newNode = new Hashtable();
        newNode.put("node.name", anetAddr);

        newNode.put("node.LLAddress", addr);

        newNode.put("node.physicalHost", addr);

        newNode.put("node.neighbor", "no");
      
        nodelist.put(addr, newNode);
      } 
      Four11Server.sendBye("(disconnect)");
    }

    return anetAddr;
  }



  /**
   * translates a virtual Active Node name to a link layer Address,
   * currently IP.
   * @param nodeName the name of the Active Node
   * @return a String representing the link layer address
   */
  protected synchronized static String getLinkAddress(String nodeName) {

    Hashtable nodelist = (Hashtable) NodeProperties.get("network.nodes");
    Hashtable nodeProps = null;
    String addr = null;
    try {
      nodeProps = (Hashtable) nodelist.get(nodeName);
      addr = (String) nodeProps.get("node.LLAddress");
    } catch (NullPointerException e) {
      // not in cache, get it from Four11 server
      String ourNetName = (String) NodeProperties.get("node.net");

      Hashtable newNode = new Hashtable();
      newNode.put("node.name", nodeName);

      Four11Server = new Four11Stub();
      int id = Four11Server.getID();
      // get this node's IP address
      String Query = "(net: " + ourNetName + " (node: " + nodeName + " IP: *))";
      addr = QueryFour11Server(Query);
      newNode.put("node.LLAddress", addr);

      Query = "(net: " + ourNetName + " (node: " + nodeName + " host: *))";

      String Reply = QueryFour11Server(Query);
      newNode.put("node.physicalHost", Reply);

      newNode.put("node.neighbor", "no");
      
      nodelist.put(nodeName, newNode);
      Four11Server.sendBye("(disconnect)");
    }

    return addr;
  }

  /**
   * get the port that a Active Node is listening to
   *
   * @param nodeName the name of the Active Node
   * @return the port it is listening to
   */
  protected static int getPort(String nodeName) {
   // find a UDP port using the node name 
   int portNum = 0;
   try {
    Hashtable nodelist = (Hashtable) NodeProperties.get("network.nodes");
    Hashtable nodeProps = (Hashtable) nodelist.get(nodeName);
    String port = (String) nodeProps.get("node.port");
    portNum = Integer.decode(port).intValue();
   } catch (Exception e) {
     Thread.currentThread().stop();
   }
   return portNum;
  }

  /**
   * get link bandwidth to neighbor
   *
   * @param nodeName name of neighbor
   * @return the link bandwidth (in Kbps) if link exists else 0.
   */
  protected static int getLinkBandwidthToNode(String nodeName) {
    // first test to see if link exists to node (ie. if neighbor)
    Hashtable nodeList = (Hashtable) NodeProperties.get("network.nodes");
    Hashtable nodeProps = (Hashtable) nodeList.get(nodeName);

    if (((String) nodeProps.get("node.neighbor")).equals("yes")) {
      return (Integer.parseInt((String) nodeProps.get("node.linkBandwidth")));
    } else {
      return 0;
    }
  }
  
  /**
   * get the link layer address for Active Node <deprecated>
   *
   * @param nodeName  name of the node
   * @return a String containing the link layer address
   */
  protected static String getLLAddress(String nodeName) {
    Hashtable nodeList = (Hashtable) NodeProperties.get("network.nodes");
    Hashtable nodeProps = (Hashtable) nodeList.get(nodeName);
 
    return (String) nodeProps.get("node.LLAddress");
  }

  /**
   * accepts a byte array representing an envinroment specific SmartPacket 
   * and sends it to the specified Active Node  by calling a PortManager's send method
   *
   * @param addr  the IP address of the Active Node
   * @param port the port to which to send the packet
   * @param packetBytes  a byte array containing bytes of the SmartPacket
   */
  protected synchronized static boolean transmit(InetPacket ipkt) {
    SPThread spt = SPThread.currentSPThread();
    KUSmartPacketV2 kusp = null;
    int PacketID = 0;
    if (spt != null) {
      kusp = spt.getSmartPacketforSPThread();
      if (kusp != null) {
        PacketID = kusp.PacketID;
      } else {
        PacketID = 0;
      }
    }
    // System.err.println("Packet " + PacketID + " adding addr " + ipkt.getIPAddress());
    transmitQ.insert(ipkt);
    // System.err.println("Packet " + PacketID + " finished adding pkt" );
    if (EventMgr.traceDepthSupported(TR_SYSTEM)) {
      int port = ipkt.getPortNumber();
      byte[] packetBytes = ipkt.getPacketBytes();
      String sbuf = "PORT=" + String.valueOf(port) + " PACKETSIZE=" + packetBytes.length;
      EventMgr.postEvent(TR_SYSTEM, "ActiveNodeManager.PortInterface.PacketSent", System.currentTimeMillis(), 0, sbuf);
 
      sbuf = "PORT=" + String.valueOf(port) + " TRANSMIT_Q_SIZE=" + transmitQ.size();
      EventMgr.postEvent(TR_SYSTEM, "ActiveNodeManager.PortInterface.PacketsPending", System.currentTimeMillis(), 0, sbuf);
    } 

    return true;
  }


  /**
   * accepts a byte array representing an envinroment specific SmartPacket 
   * and sends it to the specified Active Node  by calling a PortManager's send method
   *
   * @param addr  the IP address of the Active Node
   * @param port the port to which to send the packet
   * @param packetBytes  a byte array containing bytes of the SmartPacket
   * @exception IOException if packet could not be sent to given address
   */
  protected void sendPacket(String addr, int port, byte[] packetBytes) throws IOException {
    if (EventMgr.traceDepthSupported(TR_SYSTEM)) {
      String sbuf = "IPADDR= " + addr + " PORT=" + String.valueOf(port);
      EventMgr.postEvent(TR_SYSTEM, "ActiveNodeManager.PacketProcessing.GetIPAddress", System.currentTimeMillis(), 0, sbuf);
    }

    // get the InetAddress for the node
    InetAddress iaddr = null;
    try {
        iaddr = InetAddress.getByName(addr);
    } catch (UnknownHostException e) {
      // System.err.println("PortInterface: Unknown address " + addr);
       return;
    }


    // find portmanager
    DatagramPacket DP = null;
    try {
      PortManager portMgr = (PortManager) ActiveNode.getPortManagerForPort(port);
      DP = new DatagramPacket(packetBytes, packetBytes.length, iaddr, port);
    
      portMgr.send(DP);
      bytesSent+=packetBytes.length; /* SB added */
      packetsSent++;
      
      // log event 
      if (EventMgr.traceDepthSupported(TR_SYSTEM)) { 
        String sbuf = "PORT= " + String.valueOf(port) + " PACKETSIZE=" + String.valueOf(packetBytes.length);
        EventMgr.postEvent(TR_SYSTEM, "PortInterface.ThroughputStats.PacketsSent", System.currentTimeMillis(), 0, sbuf);
      }
      
    } catch (NullPointerException e) {
      // if port mgr is null
      e.printStackTrace();
    } catch (IOException ioe) {
      ioe.printStackTrace();
      // get ANaddress given Ip address
      String anode = null;
	System.out.println("Packet Length: "+packetBytes.length); /* SB */
      Hashtable nodelist = (Hashtable) NodeProperties.get("network.nodes");
      for (Enumeration nodes = nodelist.elements(); nodes.hasMoreElements(); ) {
        Hashtable nodeInfo = (Hashtable) nodes.nextElement();
     
        if (nodeInfo.get("node.LLAddress").equals(addr)) {
          anode = (String) nodeInfo.get("node.name");
          break;
        }
      }

      RoutingMgr.defaultRouting().PortDown(anode);
    }
    
    return;
  }


  /**
   * Decides if a particular class needs to be transmitted
   * @param classType class of object
   *
   */
  private static boolean needsTransmission (Class classType) {
    if (classType.isPrimitive()) {
      return false;
    }
    String className = classType.getName();
    // No need to send java classes
    if (className.startsWith ("java.") || className.startsWith("magician.")) {
      return false;
    }
    // Don't send the ActiveNodeManager 
    if (primitiveClasses.contains(className)) {
      return false;
    }

    return true;
  }


  /**
   * Routine to send SmartPackets.
   * @param PortName name of destination active node
   * @param SPObject the SmartPacket object
   * @return true if transmission successful, false otherwise
   */
  protected synchronized boolean SendSmartPacket (String PortName, Object SPObject) {
     
    // If the packet is destined for us, don't send
    if (PortName.equals (ourNodeName)) {
      // System.err.println("PortInterface: Same dest. as host");
      return false;
    }

    OnTheWirePacket OTWP = findOTWPFor((KUSmartPacketV2) SPObject);
    int pktid = ((KUSmartPacketV2) SPObject).PacketID;
    
    if (EventMgr.traceDepthSupported(TR_PACKET)) {
      String sbuf = "NODE=" + PortName;
      EventMgr.postEvent(TR_PACKET, "PortInterface.PacketProcessing.SendBaseKUSP", 
                       System.currentTimeMillis(), pktid, sbuf);
    }
			 
    // Send the object byte array to the output module
    String ipAddr = getLinkAddress(PortName);
    int port = getPort(PortName);
    byte[] packet = OTWP.WritePacket();
    InetPacket ipkt = new InetPacket(ipAddr, port, packet);
    boolean trCode = transmit(ipkt);

    // remove the OTWP entry from the Cache    
    OnTheWirePacket.destroyOTWP(pktid);
    
    OTWP = null;
    return trCode; 
  }


  /**
   * create ANEP encapsulation for SP object
   *
   * @param SPObject the Smartpacket object
   * @return an ANEP encapsulation of the SmartPacket
   *
   * @see OnTheWirePacket
   */
  protected static OnTheWirePacket findOTWPFor(KUSmartPacketV2 SPObject) {

    OnTheWirePacket OTWP = null;

    // Find the cached packet based on its ID
    try {
      if (EventMgr.traceDepthSupported(TR_MEASURE)) { 
        EventMgr.postEvent(TR_MEASURE, "PortInterface.Packetprocessing.FindOTWPStart", 
                         System.currentTimeMillis(), ((KUSmartPacketV2) SPObject).PacketID);
      }
			 
      OTWP = OnTheWirePacket.getOTWP(SPObject.PacketID);
      
      if (EventMgr.traceDepthSupported(TR_MEASURE)) { 
        EventMgr.postEvent(TR_MEASURE, "PortInterface.Packetprocessing.GetOTWP", 
                         System.currentTimeMillis(), ((KUSmartPacketV2) SPObject).PacketID);
      }
			 
      OTWP.setObject (SPObject);
      
      if (EventMgr.traceDepthSupported(TR_MEASURE)) { 
        EventMgr.postEvent(TR_MEASURE, "PortInterface.Packetprocessing.SetOTWPObject", 
                         System.currentTimeMillis(), ((KUSmartPacketV2) SPObject).PacketID);
      }
			 
    } catch (IOException e) {
        e.printStackTrace();
    } catch (NullPointerException t) {
      
      if (EventMgr.traceDepthSupported(TR_MEASURE)) { 
        EventMgr.postEvent(TR_MEASURE, "PortInterface.Packetprocessing.CreateOTWPStart", 
                  System.currentTimeMillis(), ((KUSmartPacketV2) SPObject).PacketID);
      }
			 
        OTWP = createNewOTWPacket(SPObject);
	
      if (EventMgr.traceDepthSupported(TR_MEASURE)) { 
        EventMgr.postEvent(TR_MEASURE, "PortInterface.Packetprocessing.CreateOTWPEnd", 
	               System.currentTimeMillis(), ((KUSmartPacketV2) SPObject).PacketID);
      }
        try {
          OTWP.setObject(SPObject);
        } catch (IOException ioe) {
          ioe.printStackTrace();
        }
			 
    }

    return OTWP;
  }

  /**
   * extract executable SmartPacket object from ANEP encapsulation
   *
   * @param packetbytes bytes of the ANEP packet
   * @return the executable Smartpacket object
   * @exception NullPointerException if executable cannot be created
   */
  protected static KUSmartPacketV2 getExecutable(byte[] packetbytes) 
    throws NullPointerException {
    Thread thr = Thread.currentThread();
    int prio = thr.getPriority();
    thr.setPriority(prio - 1);
    OnTheWirePacket OTWPacket = null;

    // Read in an object and check its data
    try {
      OTWPacket = new OnTheWirePacket(packetbytes);
      packetbytes = null;
    } catch (StreamCorruptedException e) {
       System.err.println("Packet Assembly Failed");
       // log failure as event
      if (EventMgr.traceDepthSupported(TR_MEASURE)) { 
        EventMgr.postEvent(TR_PACKET, "ActiveNodeManager.PacketProcessing.AssemblyFailed", System.currentTimeMillis(), (long) 0);
      }
       thr.setPriority(prio);
       throw new NullPointerException();
    }
    thr.setPriority(prio);
    return OTWPacket.SPObject;
  }


  /**
   * Sends an extra packet for embedded classes
   *
   * @param OTWP the OnTheWirePacket object being created
   * @param ClassToSend the embedded class being written out
   * @exception ClassNotFoundException if class is not found
   * @exception IOException if class cannot be loaded
   *
   * @see OnTheWirePacket
   */
  private static void SendClass (OnTheWirePacket OTWP, Class ClassToSend) 
      throws ClassNotFoundException, IOException {

    byte[] ClassData = SPLoader.localLoadClass (ClassToSend.getName ());
    OTWP.addClassDef (ClassToSend.getName(), ClassData);
  }


  /**
   * creates a Type definition for the incoming class
   * @param cl the class whose type is being defined
   * @return a 16 byte array representing the Type ID
   */
  protected static byte[] defineType(KUSmartPacketV2 SPObject) {

    String superClass = null;
    Class cl = SPObject.getClass();
    Class spClass = cl;

    while (! spClass.getSuperclass().getName().equals("magician.Node.KUSmartPacketV2") &&
           ! spClass.getSuperclass().getName().equals("magician.Node.ReliableCommFW")) {
      // get the super class of this type to get consistent type def
      //System.err.print("PortInterface: " + spClass + " has super class ");
      spClass = spClass.getSuperclass();
      //System.err.println(" " + spClass);
    }
  
    superClass = spClass.getName();

    byte TypeID[] = (byte[]) TypeIDTable.get(superClass);

    if (TypeID != null) 
      return TypeID;
    else if (SPObject.TypeID != null) {
      TypeIDTable.put(superClass, SPObject.TypeID);
      return SPObject.TypeID;
    }    
 
    // set the type id to be the hash of the bytes of the class definition of the super class
    byte[] ClassData = null;
    SmartPacketLoader spl = null;
    try {
      try {
        // spl = (SmartPacketLoader) Class.forName(superClass).getClassLoader();
        spl = (SmartPacketLoader) spClass.getClassLoader();
        ClassData = spl.localLoadClass(superClass);
      } catch (NullPointerException npe) {
        ClassData = SPLoader.localLoadClass(superClass);
      } catch (ClassCastException cce) {
        ClassData = SPLoader.localLoadClass(superClass);
      }
        
      if (ClassData == null) System.err.println("PortInterface: class bytes not found");
      TypeID = MessageDigest.getInstance(HASHALG).digest(ClassData);
    } catch (NoSuchAlgorithmException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
        System.out.println("ActiveNodeManager: Class not found " + cl.getName());
	// temporary measure to get TypeID
        TypeID = new byte[16];
        String objString = superClass.getClass().toString();
        System.arraycopy(TypeID, 0, objString.getBytes(), 0, 16);
    }
    TypeIDTable.put(superClass, TypeID);
    return TypeID;
  }

  /**
   * creates a new OnTheWirePacket given the executable object
   * @param destination the destination of the SmartPacket
   * @param SPObject the executable SmartPacket Object
   * @return a new OnTheWirePacket object
   *
   * @see OnTheWirePacket
   */
  protected synchronized static OnTheWirePacket createNewOTWPacket (KUSmartPacketV2 SPObject) {
    // Create a new OnTheWirePacket to send
    OnTheWirePacket OTWP = null;
    Class objclass = SPObject.getClass();
    
    //   if (needsTransmission (objclass)) {

      try { 
        String ClassName = objclass.getName ();
        byte[] TypeID = null;
        if (SPObject.TypeID == null) {
          SPObject.TypeID = defineType(SPObject);
        }
        TypeID = SPObject.TypeID;

        OTWP = new OnTheWirePacket (KU_TYPEID, TypeID, SPObject);

        // If the source address is not set, set it
        String Addr = ((KUSmartPacketV2) SPObject).Source_Address;
        if (Addr  == null) {
	  ((KUSmartPacketV2)SPObject).Source_Address = ourNodeName;
          Addr = ourNodeName;
	}
        
        OTWP.addSourceDef(Addr);

        // If the destination address is not set, set it to the port name
        Addr = ((KUSmartPacketV2) SPObject).Destination_Address;
        OTWP.addDestinationDef(Addr);

        // OTWP.setObject(SPObject);
        OTWP.SPObject = SPObject;
        OTWP.SPObject.PacketID = OTWP.PacketID;
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
 // }
    return OTWP;
  }


  /**
   * cleans up before exiting
   */
  public void finalize() {
    Four11Server.send("(disconnect)");
  }
}

