/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
package magician.Node;

import java.io.*;
import java.net.*;
import java.lang.reflect.*;
import java.util.*;
import java.security.*;
import magician.PerfTools.*;

/**
 * The PortManager is responsible for sending and receiving Active Network Encapsulation Protocol
 * (ANEP) packets over a UDP port. Each PortManager manages one UDP port. Received packets
 * are forwarded to the Demultiplxer by putting them on a queue read by the Demultiplexer. ANEP 
 * packets are sent over the network by calling the send() method  of the PortManager.
 *
 * @see Demultiplexer
 * @see Queue
 */

public class PortManager extends DatagramSocket implements Runnable, PerfConstants {
  static int MAX_DATAGRAM_PACKET_LEN = 64*1024;  // corr. to an int
  
  // bandwidth of the link to the neighboring port in Kbps
  int linkBandwidth = 0;

  /**
   * the demultiplexer queue on which incoming packets will be placed
   */
  public Queue DemuxQ;	/* SB made public */

  /**
   * the port 
   */
   public int port; /* SB changed to public */

  /**
   * datagram packet instance and its data buffer. The datagram packet instance
   * is re-initialized for every invocation of the receive method
   */
   DatagramPacket DGPacketRecv;
   byte[] DGpacketData;
  
  /**
   * count of packets seen for statistics purposes
   */
   public int packetcount = 0; /* SB changed to public */
   public int bytesReceived = 0; /* SB added */

  /**
    * Eventmanager to report events to
    */
  EventManager evm;

  /**
   * creates a new manager for the specified UDP port, list of names of virtual active nodes
   * connected to it. The Portmanager receives datagram packets, extracts out the ANEP encapsulation
   * from the data and places the ANEP packet on the demultiplexer queue.
   *
   * @param listenPort   the UDP port on which this port manager receives Smartpackets
   * @param neighbors  a Hashtable containing the names of the neighboring  nodes and their physical hosts
   *
   * @see java.util.Hashtable
   * @see Queue
   */
  public PortManager(int listenPort) throws SocketException {
    super(listenPort);
    port = listenPort;
    DGpacketData = new byte[MAX_DATAGRAM_PACKET_LEN];
    DGPacketRecv = new DatagramPacket(DGpacketData, MAX_DATAGRAM_PACKET_LEN);

  }

  // Returns the bandwidth of the link to the neighboring port
  public int getLinkBandwidth() {
    //    System.out.println("link bw = " + linkBandwidth);
    return linkBandwidth;
  }


  /** 
   * retrieves SmartPackets encapsulated inside ANEP protocol arriving over UDP port
   * @returns an array of bytes comprising an ANEP encapsulated packet
   */
  public byte[] receive() throws IOException {
    // receive datagram using default mechanism 

    super.receive(DGPacketRecv);
    // truncate to fit exact data size
    int packetlength = DGPacketRecv.getLength();
    try {
      if (evm.traceDepthSupported(TR_SYSTEM)) {
      String sbuf = "PORT=" + String.valueOf(port) + " PACKETSIZE= " + String.valueOf(packetlength);
      evm.postEvent(TR_SYSTEM, "PortManager.PacketReceive.Port", System.currentTimeMillis(), 0, sbuf);
      }
    } catch (NullPointerException npe) {
      try {
        evm = (EventManager) ActiveNode.getEventManager();
        if (evm.traceDepthSupported(TR_SYSTEM)) {
        String sbuf = "PORT=" + String.valueOf(port) + " PACKETSIZE= " + String.valueOf(packetlength);
        evm.postEvent(TR_SYSTEM, "PortManager.PacketReceive.Port", System.currentTimeMillis(), 0, sbuf);
        }
      } catch (NullPointerException npe2) {
        // Event manager still not created
        // oh well, next time
      }
    }

    packetcount++;
    bytesReceived+=packetlength; /* SB added */

    byte[] packet = new byte[packetlength];
    System.arraycopy(DGpacketData, 0, packet, 0, packetlength);
 
    // reinitialize datagram packet for reuse
    // DGpacketData = new byte[MAX_DATAGRAM_PACKET_LEN];
    DGPacketRecv.setData(DGpacketData);
    DGPacketRecv.setLength(MAX_DATAGRAM_PACKET_LEN);
    return packet;
  }

  // Port handling routine.  This routine manages a single port
  public void run() {
    // wait for other parts of the system to set up.
    try {
      Thread.currentThread().sleep(500);
    } catch (InterruptedException ie) {
      ie.printStackTrace();
    }

    while (true) {
      try {
          // loop and read packets coming on assigned UDP port
	  byte[] packetData = this.receive();  // method blocks until packet is received
          
          // cart off packet to demux by placing it on the Demux queue.	
          // the port on which it came on
          if (evm.traceDepthSupported(TR_SYSTEM)) {
          String sbuf = "PORT=" + String.valueOf(port) + " PACKETSIZE= " + String.valueOf(packetData.length);
          evm.postEvent(TR_SYSTEM, "PortManager.PacketReceive.InsertDemuxQ", System.currentTimeMillis(), 0, sbuf);
          }

          if (packetData == null) throw new IOException();

          Demultiplexer.queuePacket(port, packetData);
      } catch (IOException e) {
        // System.err.println("PortManager: " + e + " on " + port);
        continue;
      } catch (NullPointerException npe) {
        evm = (EventManager) ActiveNode.getEventManager();      
      }  
    }
  }
}


