/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/


package magician.Node;

import java.io.*;
import java.net.*;
import java.util.*;
import magician.PerfTools.*;

/**
 * Implements the RIP routing protocol for the discovery of neighboring
 * active nodes. It implements the <tt>RoutingProtocol</tt> interface so
 * that this protocol can be "stitched" in with the <tt>RoutingMgr</tt>'s
 * list of available protocols. In fact, this is the default routing protocol
 * for this implementation of the active network.
 * The RIPRouting protocol maintains an internal routing table which is implemented
 * as a vector of type <tt>TableEntry</tt>, which contains information about the 
 * reachable nodes in the network and their hop distance from current node.
 *
 * @see RoutingMgr
 * @see TableEntry
 */
public class RIPRouting implements RoutingProtocol, PerfConstants {
  private static int sleepTime = 10000; // default is 30s
  private static int infinity= 16;   
  private static Vector table = new Vector(); /* the router table */
  private static Vector threads= new Vector(); /* list of threads */
  private static String localaddr;


  /**
   *  Event manager to log events
   */
  private static EventManager EventMgr;

  /**
   * initializes the internal routing table and registers protocol with the Routing Manager
   */
    public RIPRouting () {
      RoutingMgr.Register(this);
      EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");
    }
   
  /**  
   * called by the routing manager to inform the protocol of a new neighbor
   * @param hadr the name of the neighboring active node
   */
   public void PortUp(String hadr){

     try {
       Thread.currentThread().sleep(3000);
     } catch (Exception se) {
       System.err.println(se);
     }
     localaddr = ActiveNodeManager.GetNodeName();
     // System.err.println("Port UP: " + localaddr+" "+hadr);
     /* also add it as an entry in the router table */
     TableEntry tmpobj1= new TableEntry();

     tmpobj1.setnexthop(localaddr);   //add itself in the tble
     tmpobj1.laddr= localaddr;        // with a metric of 0

     tmpobj1.Source_Address = localaddr;
     tmpobj1.setmetric(0);
     tmpobj1.settimer(0);         //just added
     if(table.size()==0){         //if this is the first time
           table.addElement(tmpobj1);  // add it to the router table
           TimeThread timethread = new TimeThread();
           timethread.timer = sleepTime; //update every 10 seconds
           timethread.start(); //start the timer
     }

     int index= Findelement(hadr);
     if ( index != -1){
          TableEntry found = (TableEntry) table.elementAt(index);
           if(found.getmetric()==1)
              return;
     }
     TableEntry tmpobj= new TableEntry(); 
     tmpobj.setnexthop(hadr);   //connected directly
     tmpobj.laddr= hadr; 
     tmpobj.Source_Address = localaddr; 
     tmpobj.setmetric(1);
     tmpobj.settimer(0);         //just added

     /*start a thread that will keep sending updates on that port */
     SendThread th = new SendThread(sleepTime);
     threads.addElement(th);
     int j=  threads.size();
     /* sleep time is 30 seconds*/
     ((SendThread)(threads.elementAt(j-1))).timer= sleepTime;
     /* object output stream, or now portname */ 
     ((SendThread)(threads.elementAt(j-1))).portname=hadr;
     /* run the thread */
     ((SendThread)(threads.elementAt(j-1))).start();

     Delelement1( hadr); 	 //deletes it if it is there
     System.err.println("(PortUp) deleting: "+hadr);
     //Addentry(tmpobj);  // add it to the router table 
     table.addElement(tmpobj);

     return;

   }
   
   /**
    * called when node is no longer reachable 
    * @param PortName name of down active node
    */
   public void PortDown(String PortName) {
       System.err.println("RIPRouting: Port " + PortName + " down");
       //find the specified down thread 
       for(int j=0;j<threads.size();j++) {
            SendThread sth= (SendThread) threads.elementAt(j);
            if(sth.portname.equals(PortName)) { // if found
                 sth.stop();  //stop it
                 threads.removeElement(sth);   //remove it from the list
            }
       }
       for( int k=0;k<table.size();k++){
            TableEntry tblent=(TableEntry) table.elementAt(k);
            if(tblent.getnexthop().equals(PortName)){
     		  System.err.println("(PortDown) deleting: "+tblent.laddr);
                  Delelement(tblent.laddr);
                  k--;
            }
       }
   }
   

    /**
     * get next hop (called by a smartpacket that wants 
     * to reroute itself) 
     *
     * @param iaddr the destination to which the next hop is desired
     * @return the name of the next hop
     * @exception NoRouteToHostException if no route exists or node is unreachable
     */
    public static String getNextHop(String iaddr) throws NoRouteToHostException {

      int ind = Findelement(iaddr);  //find the dest in the table
      if (ind == -1) {    // return null if not found
        System.err.println("(RIPRouting:getNextHop) destination " + iaddr + "was not found");
        throw new NoRouteToHostException("No route to " + iaddr);    
      }
      //entry for destination was found in this router table
      //MySystem.println(" nexthop was found");
      TableEntry tmpentry = (TableEntry) table.elementAt(ind);
      String raddr = tmpentry.getnexthop(); //what is it
      //System.out.println("RoutingMgr: getnexthop is " + raddr);
      return raddr;        // return nexthop to dest
    }


  /**
   * get next hop interface of Routing protocol
   * @param destination the name of the destination active node
   * @exception NoRouteToHostException if no route exists or node is unreachable
   */
  public String getnexthop(String destination) throws NoRouteToHostException {
    return getNextHop(destination);
  }

  /**
   * returns a copy of the routing table
   * @return the routing table
   * @see TableEntry
   */
   protected static Vector gettable() {
          return table;
   }


  /**
   * updates the metric in a router table
   * @param index the index of the entry in the routing table
   * @param mtrc the number of hops
   * @param fst name of gateway
   */
  protected static void Update(int index, int mtrc, String fst) {
 
        /* update */
        ((TableEntry)(table.elementAt(index))).metric = mtrc;
        ((TableEntry)(table.elementAt(index))).first_gateway= fst;         

        /* now those prints are for testing only, to be deleted later */
        TableEntry entr= (TableEntry)table.elementAt(index);
        MySystem.println("Entry Updated ");
        entr.printDetails();
	// SFB
  }

  /**
   * adds a new entry to the router table
   * @param entr TableEntry object denoting new entry in the table
   * @see TableEntry
   */
  protected static void Addentry(TableEntry entr) {
      // log event
      if (EventMgr.traceDepthSupported(TR_PACKET)) {
        String sbuf = "TABLE_SIZE=" + String.valueOf(table.size());
        EventMgr.postEvent(TR_PACKET, "RoutingMgr.RoutingTable.Size", System.currentTimeMillis(), 0, sbuf);
      }
      /* add */
      table.addElement(entr);

      String port;
      /* print to test, delete later */
      // MySystem.println("New Entry Added ");
        entr.printDetails();
 
      /* use triggered updates, when a new entry is added inform other
        nodes connected to me about it */
      for(int k=0;k<threads.size();k++) {
          port= ((SendThread)(threads.elementAt(k))).portname;
          entr.Destination_Address = port;
          // System.err.println("RIPRouting: sending tableentry pkt to " + port + " Source = " + entr.Source_Address);
          ActiveNodeManager.SendSmartPacket(port, entr);
      }
  }      

    /**
     * delete the entry for the active node from the routing table
     * @param adr name of active node to be deleted
     */
    protected static void Delelement( String adr) {
      // log event
      if (EventMgr.traceDepthSupported(TR_PACKET)) {
      String sbuf = "TABLE_SIZE=" + String.valueOf(table.size());
      EventMgr.postEvent(TR_PACKET, "RoutingMgr.RoutingTable.Size", System.currentTimeMillis(), 0, sbuf);
      }
      String port;
      int ind = Findelement(adr);
      if(ind == -1) return;
      table.removeElementAt(ind);
      
      /* print info, just for testing, to be deleted later */

      //create a new smartpacket and send it to nieghbors */
      DeleteEntry delentry = new DeleteEntry(adr,localaddr);

      System.err.println("Element Deleted " + adr );

      /* use triggered updates, when an entry is deleted inform 
      other nodes connected to me about it */
      for(int k=0;k<threads.size();k++) {
          port= ((SendThread)(threads.elementAt(k))).portname;
          delentry.Destination_Address = port;
          ActiveNodeManager.SendSmartPacket(port, delentry);
      }
      /* also remove every path through that removed node */
      for( int l=0;l<table.size();l++){
          TableEntry tblent=(TableEntry) table.elementAt(l);
          if(tblent.getnexthop().equals(adr)){
               table.removeElementAt(l);
               MySystem.println("Element Deleted " + l );
               l--;
          }
       }

    }     

    /**
     * delete the entry for the active node from the routing table <deprecated>
     * @param adr name of active node to be deleted
     */
    protected static void Delelement1( String adr) {
      // log event
      if (EventMgr.traceDepthSupported(TR_PACKET)) {
        String sbuf = "TABLE_SIZE=" + String.valueOf(table.size());
        EventMgr.postEvent(TR_PACKET, "RoutingMgr.RoutingTable.Size", System.currentTimeMillis(), 0, sbuf);
      }

      //this delete is different then the previous one in that it just delete
      //one element no more, this one is called when a shorter path is found
      //not when a node(or link) in declared one (that's the previous) 
      //Delelemt.  
      int ind = Findelement(adr);
      if(ind == -1) return;
      table.removeElementAt(ind);
    
      /* print info, just for testing, to be deleted later */
      MySystem.println("Element Deleted " + ind );
    }

   /**
    * set timer of an entry in the table 
    * @param adr name of active node whose entry is to be changed
    * @param flg the value of the timer
    */
    protected static void SetFlag (String adr, int flg) {

       int ind = Findelement(adr);
       if(ind == -1) return;
       ((TableEntry)(table.elementAt(ind))).settimer(flg); 
    }

   /**
    * find a table entry in the router table
    * @param adr name of active node
    * @return index of table entry
    */
    protected static int Findelement( String adr ) {
        int index=-1;
        for(int i=0;i<table.size(); i++) {
            TableEntry ent = (TableEntry)table.elementAt(i);
            if(ent.laddr.equals( adr) ) 
                  return i;
        }

        return index;
  
    }
  
    /**
     * Send the router table to given active node
     * @param portname name of active node
     */
    protected static void SendTable(String portname){

	//System.out.println ("Sending Table...");

       /* send the router table */
       int num=table.size();
       for(int i=0; i<num ;i++) {

	  //System.out.println ("Sending element " + i + ", num=" + num);
	 try {
          TableEntry trm = (TableEntry)table.elementAt(i);
          //trm.sendentry(portname); //send it as a smartpacket
          trm.Destination_Address = portname;
          // System.err.println("RIPRouting sending Tableentry pkt to " + portname  + "Source = " + trm.Source_Address);
          ActiveNodeManager.SendSmartPacket(portname, trm);

          /*debugging */ 
          //MySystem.println("Object Sent to : "+portname);
          //trm.printDetails();
         /* now we also need to check the timer, add one to it, if it is
          more than 10, it means than no updates happened for more
          than 3 minutes and we need to delete it*/
           /* now this checks on the timer if it was not updated in the 
            last 3 minutes, delete it */
/*
           if(trm.gettimer() > 180 ) {
               if(trm.getmetric() > 0){ //if it is not the local node
		    MySystem.println("(SendTable) to be deleted");
                    trm.printDetails();
                    Delelement( trm.laddr);
                    num--;
                    i--;
		}
           }
*/
         } catch (ArrayIndexOutOfBoundsException e) {
            continue;
         }
       }

   }

   /**
    * Check if an entry is in the table
    * @param ten the table entry to be checked
    */
    protected synchronized static void CheckEntry(TableEntry ten) {
       //MySystem.println("I am here");
       ten.printDetails();
       if(ten.getnexthop().equals(localaddr)) {
            //MySystem.println("Bye");
	    return;
       }

       int ind = Findelement(ten.laddr);
       if(ind == -1 && ten.getmetric() < infinity){
            //MySystem.println("Entry not in table");
            //if this is a new element add it
            TableEntry tmpentr = new TableEntry();
            tmpentr.laddr = ten.laddr;
            tmpentr.setnexthop(ten.Source_Address);
            tmpentr.Source_Address = localaddr;
            tmpentr.setmetric(ten.getmetric()+1);
            tmpentr.settimer(0);
            Addentry(tmpentr);            
            return;
       }

       //if this is already there, check the metric, update it if less
       //metric is found.
      try {
       TableEntry tmpentry = (TableEntry)table.elementAt(ind);
       if(ten.metric+1 < tmpentry.metric ) {
           //was found with a shorter path
           //just to debug
           //MySystem.println("Better path found");
           tmpentry.setnexthop(ten.Source_Address); 
           tmpentry.setmetric(ten.getmetric()+1);
           tmpentry.settimer(0);
       }
       //else it is still connected and we need to reset the
       //timer but only if same next hop plus update metric if different
       else {
         //MySystem.println("nexthop "+tmpentry.first_gateway );
         //MySystem.println("Source "+ten.Source_Address );
         if(tmpentry.first_gateway.equals(ten.Source_Address)){
              if(tmpentry.getmetric() != ten.getmetric()+1) {
                  //MySystem.println("metric updated");
                  tmpentry.setmetric(ten.getmetric()+1);
              }
              //MySystem.println("Timer Updated");
              tmpentry.settimer(0);
              //tmpentry.printDetails();
         }
       }
      } catch (Exception e) {
         System.err.println("RIPRouting: index = " + ind + " metric = " + ten.getmetric());
         e.printStackTrace();
      }
    }


    /**
     * increment timers
     */
    protected static void UpdateTimers(){
           
       /* update the timers the router table */
       int num=table.size();
       for(int i=0; i<num ;i++) {  // add 10 to every timer in the table
          TableEntry trm = (TableEntry)table.elementAt(i);
          int tim = trm.gettimer();
          if(tim <= (18*sleepTime/1000))
             trm.settimer(tim+10);
       }

    }

  /**
   * returns number of hops required to reach a given destination
   * 
   * @param laddr the destination address
   * @return number of hops required to reach it
   * @exception NoRouteToHostException if no route exists
   */
  public int getNumberOfHopsToNode(String laddr) throws NoRouteToHostException {
    int ix = Findelement(laddr);
    if ( ix < 0) throw new NoRouteToHostException();

    TableEntry ent = (TableEntry) table.elementAt(ix);

    return ent.getmetric();
  }


  /**
   * returns the routing update interval for this protocol
   * @return the update interval
   */
  public int getUpdateInterval() {
    return sleepTime;
  }
}
