/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * Copyright (c) 1997 by the University of Kansas Center for Research, Inc.
 * All Rights Reserved.  This software was developed by the Information Technology
 * and Telecommunication Center (ITTC) at the University of Kansas
 * with the assistance of DARPA.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for personal or educational use is hereby granted, provided
 * that copies of the software are not made or distributed for profit or
 * commercial advantage; that both the copyright notice and this permission
 * notice appear in all copies of the software, derivative works
 * or modified versions, and any portions thereof; that both notices appear in
 * supporting documentation; and that the use of this software is acknowledged
 * in any publications that might result from the use of this software.
 *
 * ITTC ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, AND
 * DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 *
 * RCS Identification : $Id: ResourceMgr.java,v 1.2 2000/11/02 19:14:20 kulkarni Exp kulkarni $
 *
 * @author Amit B. Kulkarni
 */


package magician.Node;

import java.util.*;
import magician.PerfTools.*;


public class ResourceMgr implements PerfConstants {
  
  /**
   * ResourceMonitor thread executes continuously to check on resource
   * consumption of active threads
   */
  protected ResourceMonitor resMon;

  /**
   *  Scheduler thread also executes continuously to schedule threads
   */
  protected Scheduler scheduler;

  /**
   * All managed spThreads belong to "Smart Packets" ThreadGroup
   */
  protected ThreadGroup threadgroup;

  /**
   * A table of reference to the small states managed by the ResourceMgr
   */
  Vector smallStates;

  /**
   * Maximum number of objects which can be created
   * by a thread
   */
  protected static long objectPerThread = 10;

  /**
   * Maximum execution time of a thread (in milliseconds) 
   */   
  protected final static long MAX_CPU_TIME = 5 * 60 * 60 * 1000;

  /**
   * Maximum number of Smart Packet which can be processed
   * simultaneously. We may want to limit this to ensure
   * timely execution of the threads
   */
  protected int maxSP;

  /**
   * initializes the ResourceMgr
   * @param maxSP the maximum number of Smartpacket threads that can run concurrently
   * @param maxObjects the maximum number of objects that can be allocated by a
 Smartpacket
   * @param execTime the maximum allowable execution time
   */ 
  public ResourceMgr(int maxSP,
		     long maxObjects, 
		     long execTime) {
    
    // set limits
    objectPerThread = maxObjects;
    // MAX_CPU_TIME = execTime;     
    this.maxSP = maxSP;
    
    // Create the Small State table
    smallStates = new Vector();

    // Create a thread group for all spThreads
    threadgroup = new ThreadGroup("Smart Packets");
    
    // Create a ResourceMonitor thread to monitor the
    // SPThreads activities. The new ResourceMonitor
    // will belong to the same ThreadGroup as the SPThreads
    // resMon = new ResourceMonitor(this);
   
    // Create the Scheduler thread
    scheduler = new Scheduler(maxSP);


    // init done
  }

  /**
   *  Allocates a new thread for the SmartPacket
   * and schedules it for execution
   * @param sp the Smartpacket object that requires execution
   */
  public void newThread(KUSmartPacketV2 sp) {
      
      try {
	SPExecEnv newSP = new SPExecEnv(sp, threadgroup,
					this);
	new Thread(newSP).start();
      } catch (ResourceException e) {
	//discard packet and enterin log
        EventManager evm = (EventManager) ActiveNodeManager.HaveService("EventManager");
        if (evm.traceDepthSupported(TR_PACKET)) {
          evm.postEvent(TR_PACKET, "ResourceManager.SPThread.Discard", System.currentTimeMillis(), sp.PacketID);
        }
      }
  }
    
  /**
   * Allocate storage for new object
   * @param className class of new object
   */
  public SPStorage newStorage (String className) {
    
    SPStorage storage = new SPStorage(className);
    return storage;
    
  }
  
}

