/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * Copyright (c) 1997 by the University of Kansas Center for Research, Inc.
 * All Rights Reserved.  This software was developed by the Information Technology
 * and Telecommunication Center (ITTC) at the University of Kansas
 * with the assistance of DARPA.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for personal or educational use is hereby granted, provided
 * that copies of the software are not made or distributed for profit or
 * commercial advantage; that both the copyright notice and this permission
 * notice appear in all copies of the software, derivative works
 * or modified versions, and any portions thereof; that both notices appear in
 * supporting documentation; and that the use of this software is acknowledged
 * in any publications that might result from the use of this software.
 *
 * ITTC ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, AND
 * DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 *
 * RCS Identification : $Id: RetransmitThread.java,v 1.1 2000/03/21 13:25:21 bushsf Exp kulkarni $
 *
 * @author Amit B. Kulkarni
 */


package magician.Node;

import java.lang.*;
import java.io.*;
import java.util.*;
import magician.PerfTools.*;
import magician.Utils.*;

public class RetransmitThread extends Thread implements PerfConstants {

  String address;
  int Port;
  byte[] pkt = null;
  int seqNo;
  int linkspeed;
  int retries;
  SliWinPortThread pThread;
  EventManager EventMgr;

  final static int TIMEOUT_PERIOD = 400;

  public RetransmitThread(SliWinPortThread parent, String IPAddr, int port, int speed, int maxTries) {
    address = IPAddr;
    Port = port;
    linkspeed = speed;
    retries = maxTries;
    pThread = parent;
    EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");
  }

  protected synchronized void setPacket(int seqnum, byte[] packet) {
    seqNo = seqnum;
    pkt = packet;
    notify();
  }

  protected synchronized void confirmPacket() {
    pkt = null;
    notify();
  }

  /**
   * keeps transmitting packet until timer expires or ack is received
   */
  public synchronized void run() {
  
   while (true) {
     while (pkt == null) {
      try {
        // wait for work to arrive
        wait();
      } catch (InterruptedException ie) {
        ie.printStackTrace();
        return;
      }
     }
   
     int rtt = (pkt.length*8*2)/(linkspeed);
     int timeout = (TIMEOUT_PERIOD > rtt)? TIMEOUT_PERIOD : rtt;

     int attempt = 0;
     while (attempt < retries) {
       if (EventMgr.traceDepthSupported(TR_PACKET)) {
       String sbuf = "PORT=" + String.valueOf(Port) + " SEQNO=" + String.valueOf(seqNo);
       EventMgr.postEvent(TR_PACKET, "ActiveNodeManager.PortThread.PacketSent", System.currentTimeMillis(), 0, sbuf);
       }
       // System.err.println("RetransmitThread: sending packet to " + address + " at time " +  System.currentTimeMillis());
       InetPacket ipkt = new InetPacket(address, Port, pkt);
       PortInterface.transmit(ipkt);

       int timePeriod = timeout * (2<<attempt);
       // int timePeriod = TIMEOUT_PERIOD * (2<<attempt);

       attempt++;

       if ((attempt%5) == 0) {
         System.err.println("RetransmitThread: sending packet for " + attempt + "th time");
       }
      
       try {
         // System.err.println("RetransmitThread: Sleeping for " + timePeriod + " ms");
         wait(timePeriod);
         if (pkt == null) break;
       } catch (InterruptedException ie) {
         // System.err.println("PortThread: woken up at time " + System.currentTimeMillis());
         // ie.printStackTrace();
         return;
       } 
     }
     
     if (attempt >= retries) {
       System.err.println("RetransmitThread: sending packet for " + attempt + "th time");
       // mark link as down and send PortDown message to Routing Mgr.
       try {
         pkt = null;
         pThread.PortDown();
       } catch (NullPointerException npe) {
         return;
       }

     } 
   }
        
  }
}
