/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * Copyright (c) 1997 by the University of Kansas Center for Research, Inc.
 * All Rights Reserved.  This software was developed by the Information Technology
 * and Telecommunication Center (ITTC) at the University of Kansas
 * with the assistance of DARPA.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for personal or educational use is hereby granted, provided
 * that copies of the software are not made or distributed for profit or
 * commercial advantage; that both the copyright notice and this permission
 * notice appear in all copies of the software, derivative works
 * or modified versions, and any portions thereof; that both notices appear in
 * supporting documentation; and that the use of this software is acknowledged
 * in any publications that might result from the use of this software.
 *
 * ITTC ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, AND
 * DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 *
 * RCS Identification : $Id: SPExecEnv.java,v 1.2 2000/10/27 16:58:28 kulkarni Exp kulkarni $
 *
 * @author Yulia Wijata, Amit B. Kulkarni
 */


package magician.Node;

import java.io.*;
import java.util.*;
import magician.PerfTools.*;

public class SPExecEnv extends Resource implements Runnable, PerfConstants {
    
    /**
     * the Smartpacket thread being monitored
     */
    protected SPThread thread;
    
    /**
     * The number of objects allocated so far by this SPThread
     */
    protected int numObjects;
    
    /**
     * Estimated memory usage (in bytes)
     * Calculated as the number of objects allocated multiplied by
     * the size of each object
     */
    protected long memUsage;
    
    /**
     * The length of execution time (in millisecond)
     * Should be less than MAX_CPU_TIME 
     */
    protected long CPUTime;
    
    /**
     * The maximum size of memory which can be allocated by
     * the SmartPacket running this thread (in bytes)
     */
    protected int totalMemory;
    
    /**
     * The I/O bandwidth that this SP can utilize (in bps)
     */
    protected int IOBandwidth;
    
    /**
     * test for thread control
     */
    protected boolean threadStarted;
    
    protected ResourceMgr rm;
    
    protected long thStartTime;
    
    protected EventManager evm;
    
    protected static Hashtable AppPredictorTable = new Hashtable();
    
    protected static Hashtable AppsRecorded = new Hashtable();
    
    protected static long clockspeed;

    protected static String cpuinfo; 

    private long finalExecTime = 0;

    static {
	/** read cpuinfo file to get machine info **/
	StringBuffer info = new StringBuffer();
	try {
	    BufferedReader freader = new BufferedReader(new FileReader("../Node/cpuinfo"));
	    clockspeed = 0;
	    String attr_value = null;
	    StringTokenizer st = null;
	    while (! (attr_value = freader.readLine()).equals("")) {
		st = new StringTokenizer(attr_value, ": \t");
		String attr = st.nextToken();
		String value = st.nextToken();
		if ((attr.equals("model_name")) ||
		    (attr.equals("processor")) ||
		    (attr.equals("bogomips"))) {
              info = info.append(value).append(":"); 
		} else if (attr.equals("cpu_MHz")) {
		    clockspeed = Float.valueOf(value).longValue();
		    info = info.append(value).append(":"); 
		}
	    }
	} catch (Exception fe) {
	    fe.printStackTrace();
	}
	cpuinfo = info.toString();
    }
    
    
    /**
     * initializes the execution environment prameters and create a new SPThread
     * @param sp the SmartPacket object
     * @param tg the thread group to which the SPThread belongs
     * @param rm the Resource Manager for the node
     * @exception ResourceException if Smartpacket's resource needs exceed global bounds
     */
    protected SPExecEnv(KUSmartPacketV2 sp, ThreadGroup tg, ResourceMgr rm)
	throws ResourceException {
	super(sp);
	this.rm = rm;
	
	if (evm == null) {
	    evm = (EventManager) ActiveNodeManager.HaveService("EventManager");
	}
	
	// set the resource requirements
	totalMemory = sp.Memory_Requirements;
	IOBandwidth = sp.IO_BW_Requirements;
	
	
	/*
	 * alternatively, read from predictor object
	 */ 
	long avgTime = 0;
	long variance = 0;
	String appname = spOwner.getClass().getName(); 
	
	
	/*
	 * record app start time
	 */
	recordStartTime(evm, appname);
	
	predictor app_pred = null;
	
	long meanTime = 0;
	try {
	    app_pred = getPredictorForApp(appname);
	    
	    // set times based on values from predictor object
	    app_pred.setFidelity(predictor.HIGH_FIDELITY);
	    prediction p = app_pred.getMean();
	    meanTime =  (long)(p.getValue()*100/(100+p.getError()));
	    p = app_pred.getPercentile(99);
	    avgTime = (long)(p.getValue()*100/(100+p.getError()));
	    // compute expected cpu time for this packet
	    CPUTime = avgTime ;
	    
	    // get variance
	    p = app_pred.getStdDev();
	    long stddev = (long)(p.getValue()*100/(100+p.getError()));
	    variance = stddev * stddev;
	} catch (FileNotFoundException fnfe) {
	    long sp_cpu = sp.CPU_Requirements;
	    long max_cpu = ResourceMgr.MAX_CPU_TIME;
	    //System.err.println("sp_cpu = " + sp_cpu + ", max_cpu = " + ResourceMgr.MAX_CPU_TIME);
	    if (sp_cpu > 0) {
		CPUTime = (sp_cpu < max_cpu)? sp_cpu : max_cpu;
		avgTime = CPUTime;
		meanTime = CPUTime;
		variance = 0;
	    } else {
		CPUTime = max_cpu;
		avgTime = CPUTime;
		meanTime = CPUTime;
		variance = 0;
	    }
	} 
	
	//System.err.println("packet " + appname + " set to run for " + CPUTime);
	long now = System.currentTimeMillis();
	if (evm.traceDepthSupported(TR_PACKET)) {
	    evm.postEvent(TR_PACKET, "ResourceManager.SPThread.Allocate", now, sp.PacketID);
	}
	if (evm.traceDepthSupported(TR_USER)) {
	    String sbuf = "VALUE=" + cpuinfo;
	    evm.postEvent(TR_USER, "ActiveNode.CPUId", now, sp.PacketID, sbuf);
	    // create event from packet's package name
	    String eventname = null;
	    String pkgname = null;
	    Package pkg = spOwner.getClass().getPackage();
	    if (pkg == null) {
		pkgname = spOwner.getClass().getName();
	    } else {
		pkgname = pkg.getName();
	    }
	    eventname = pkgname + ".PacketCount";
	    evm.postEvent(TR_USER, eventname, now, sp.PacketID);
	    
	    long appStarted = ((Long)AppsRecorded.get(appname)).longValue();
	    long timediff = now - appStarted;
	    eventname = pkgname + ".AppUptime";
	    sbuf = "VALUE=" + String.valueOf(timediff);
	    evm.postEvent(TR_USER, eventname, now, sp.PacketID, sbuf);
	    
	    eventname = pkgname + ".PredictedAvgCPUTime";
	    sbuf = "VALUE=" + String.valueOf(meanTime);
	    evm.postEvent(TR_USER, eventname, now, sp.PacketID, sbuf);
	    
	    eventname = pkgname + ".PredictedCPUTime";
	    sbuf = "VALUE=" + String.valueOf(avgTime);
	    evm.postEvent(TR_USER, eventname, now, sp.PacketID, sbuf);
	    
	    eventname = pkgname + ".PredictedVarCPUTime";
	    sbuf = "VALUE=" + String.valueOf(variance);
	    evm.postEvent(TR_USER, eventname, now, sp.PacketID, sbuf);
	}
	
	threadStarted = false;
	// initialize instance variables and start thread
	numObjects = 0;
	thread = new SPThread(tg, sp, this);
	SPThread.addToGroup(thread);
	
    }
    
    /**
     * returns the number of objects allocated by the Smartpacket thread
     * @return the number of objects
     */
    public int getNumObjects() {
	return numObjects;
    }
    


    /**
     * starts SmartPacket thread and monitors cpu
     * usage.
     * Cleans up after smartpacket thread exits
     */
    public void run() {
	
	thread.setExecEnvThread(Thread.currentThread());
	/*
	 * schedule smartpacket thread
	 */
	rm.scheduler.Schedule(thread);
	
	/*
	 * wait for notfication from thread that it has started
	 */
	//	     System.err.println("SPEE " + Thread.currentThread() + " waiting for " + spOwner.getClass().getName() + " for " + CPUTime);
	
	waitForSPThread();
	
	
	/*
	 * thread has started, wait until its cpu usage
	 * runs out
	 */
	long CPUTimeUsed = 0;
	if ( !(thread.getSmartPacketforSPThread().getClass().getName().startsWith("avnmp")) &
	     !(thread.getSmartPacketforSPThread().getClass().getName().startsWith("snmp")) &
	     !(thread.getSmartPacketforSPThread().getClass().getName().startsWith("magician.Node"))) {
	    // create event from packet's package name
	    String eventname = null;
	    String pkgname = null;
	    Package pkg = spOwner.getClass().getPackage();
	    if (pkg == null) {
		pkgname = spOwner.getClass().getName();
	    } else {
		pkgname = pkg.getName();
	    }

	    long waitTime = 1000/clockspeed;
	    long waitTimeMillis = waitTime/1000000;
	    int waitTimeNanos = (int)(waitTime - 1000000*waitTimeMillis);
	    long alreadyUsed =  thread.getSmartPacketforSPThread().getExecTime();
	    long extra = 100 * CPUTime / clockspeed;
            long timeLeft = CPUTime + extra - alreadyUsed;
	    //while ((timeLeft > 0) && (thread.isAlive())) {
	    while (thread.isAlive()) {
        	try {
		    thread.join(waitTimeMillis,waitTimeNanos);
		} catch (InterruptedException ie) {
		}	
                if (thread.isAlive()) {
		    timeLeft = CPUTime + extra - thread.getSmartPacketforSPThread().getExecTime();
                }
	    }
	    if (thread.isAlive()) {
		CPUTimeUsed = thread.getSmartPacketforSPThread().getExecTime();
                System.out.println("take too long -> killed (" + CPUTimeUsed + " wasted )");
		thread.stop();
		
		long now = System.currentTimeMillis();
		if (evm.traceDepthSupported(TR_USER)) {
		    eventname = pkgname + ".PacketsTerminated";
		    evm.postEvent(TR_USER, eventname, now, spOwner.PacketID);
		    
		    eventname = pkgname + ".PacketID";
		    String sbuf = "ID=" + String.valueOf(spOwner.PacketID);
		    evm.postEvent(TR_USER, eventname, now, spOwner.PacketID, sbuf);
		    
		    eventname = pkgname + ".CPUTimeStolen";
		    sbuf = "CPU_TIME=" + String.valueOf(CPUTimeUsed/(1000*clockspeed));
		    evm.postEvent(TR_USER, eventname, System.currentTimeMillis(), spOwner.PacketID, sbuf);
		} 
		
		
		
	    } else {
		CPUTimeUsed = finalExecTime;
                // System.out.println("finish in time (" + CPUTimeUsed + ")");
                if (evm.traceDepthSupported(TR_USER)) {
		    eventname = pkgname + ".PacketsExited";
		    evm.postEvent(TR_USER, eventname, System.currentTimeMillis(), spOwner.PacketID);
                }
	    }
	    if (evm.traceDepthSupported(TR_USER)) {
		eventname = pkgname + ".AvgCPUTime";
		
		String sbuf = "AvgCPU_TIME=" + String.valueOf(CPUTimeUsed);
		evm.postEvent(TR_USER, eventname, System.currentTimeMillis(), spOwner.PacketID, sbuf);
	    }   
	}
    }
    
    
    /**
     * called by exec env to wait for thread
     */
    private void waitForSPThread() {
	
	synchronized (this) {
	    if (threadStarted == false) {
		try {
		    wait();
		    
		    thStartTime = System.currentTimeMillis();
		} catch (Exception e) {
		    e.printStackTrace();
		}
	    }
	}
    }

    /**
     * called by SmartPacket thread to notify exec env
     */
    public void notifyExecEnv() {
	
	synchronized(this) {
	    threadStarted = true;
	    try {
	        notify();
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }
    
    public void setFinalExecTime(long execTime) {
	finalExecTime = execTime;
   }
    
    private static synchronized predictor getPredictorForApp(String app) throws FileNotFoundException {
	
	predictor p = (predictor) AppPredictorTable.get(app);
  
	if (p == null) {
	    p = new predictor(app);
        AppPredictorTable.put(app, p);
        throw new FileNotFoundException();
	}
	
	return p;
    } 
    
    protected static void recordStartTime(EventManager evm, String app) {
	if (! AppsRecorded.containsKey(app)) {
          long now = System.currentTimeMillis();
          AppsRecorded.put(app, new Long(now));
	  
          String sbuf = "VALUE=" + String.valueOf(now);
          evm.postEvent(TR_USER, app + ".AppStartTime", now, 0, sbuf);
       }
    }
    
}

