/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * Copyright (c) 1997 by the University of Kansas Center for Research, Inc.
 * All Rights Reserved.  This software was developed by the Information Technology
 * and Telecommunication Center (ITTC) at the University of Kansas
 * with the assistance of DARPA.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for personal or educational use is hereby granted, provided
 * that copies of the software are not made or distributed for profit or
 * commercial advantage; that both the copyright notice and this permission
 * notice appear in all copies of the software, derivative works
 * or modified versions, and any portions thereof; that both notices appear in
 * supporting documentation; and that the use of this software is acknowledged
 * in any publications that might result from the use of this software.
 *
 * ITTC ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, AND
 * DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 *
 * RCS Identification : $Id: SPObjectOutputStream.java,v 1.1 2000/03/21 13:25:21 bushsf Exp kulkarni $
 *
 * @author Amit B. Kulkarni
 */

package magician.Node;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import magician.Utils.UnsynchVector;

public class SPObjectOutputStream extends ObjectOutputStream {

  /**
   * classes that are not transmitted over the wire
   */
  static Vector primitiveClasses;

  /**
   * the associated OnTheWirePacket object
   */
  OnTheWirePacket OTWP;


  /**
   * classes that have been described
   */
  UnsynchVector annotatedClasses;

  /**
   * Smartpacket loader for this Smartpacket type
   * @see KUSmartPacketV2
   * @see SmartPacketLoader
   */
  SmartPacketLoader SPL;


  /**
   * active node to send the serialized object
   */
  String dest;

  /**
  * initializes the output stream
  * @param out the output stream
  * @exception IOException if the object cannot be written to the stream
  */
  public SPObjectOutputStream(OutputStream out) throws IOException {
    super(out);
  }

  /**
  * initializes the output stream with additional information
  * @param out the output stream to write to
  * @param otwp the OnTheWirePacket object
  * @param SPLoader the SmartPacket loader for this type
  * @param node the destination active node
  */
  public SPObjectOutputStream (OutputStream out, OnTheWirePacket otwp, SmartPacketLoader SPLoader, String node) throws IOException {
    super(out);
    OTWP = otwp;
    SPL = SPLoader;
    dest = node;
    annotatedClasses = new UnsynchVector();
  }

    
  /**
  * describes the class on the wire so that it can be defined in another
  * active node's environment
  * @param cl class to be described
  */
  protected void annotateClass(Class cl) {
  
    String classname = cl.getName();
    try {
   
      if (needsTransmission(cl)) {
        // System.err.println("Class " + classname + " annotated");

        SendClass(cl);
      }
    } catch (NullPointerException e) {
      // primitiveClasses = new Vector();

      // initClasses();

       try {
         if (needsTransmission(cl)) {
           SendClass(cl);
           // System.err.println("Class " + classname + " annotated");
         }
       } catch (Exception t) {
          t.printStackTrace();
       }

    } catch (Exception e) {
       e.printStackTrace();
    }
  }

  /**
   * initializes the classes that should not be sent over the wire
   */
  private void initClasses() {

    primitiveClasses.addElement("KUSmartPacketV2");

    primitiveClasses.addElement("ReliableCommFW");
    primitiveClasses.addElement("StopAndWaitAck");
    primitiveClasses.addElement("Acknowledgement");
    primitiveClasses.addElement("UnsynchVector");
  }


	
  /**
   * Writes the description of the class to the output stream
   *
   * @param ClassToSend the embedded class being written out
   * @exception ClassNotFoundException
   * @exception IOException
   *
   * @see OnTheWirePacket
   */
  private void SendClass (Class ClassToSend) {
    String classname = ClassToSend.getName();
    if (annotatedClasses.contains(classname)) {
      return;
    } else {
      annotatedClasses.addElement(classname);
    }

    if (OTWP.ClassNames.get(classname) != null) {
      // class def already exists
      return;
    }
    
    try {
      Field[] memberFields = ClassToSend.getFields ();
      for (int i= 0; i < memberFields.length; ++i) {
	Class classType = memberFields[i].getType();
        if (classType.isArray()) {
	  classType = classType.getComponentType();
	}
        // If the class needs to be sent, send it
	if (needsTransmission (classType)) {
	  SendClass (classType);
	}
      }
      // If the class also needs to be transmitted, put it into packet
      
      byte[] ClassData = SPL.localLoadClass (classname, dest);
      OTWP.addClassDef (classname, ClassData);
    } catch (CloneNotSupportedException e) {
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Decides if a particular class needs to be transmitted
   * @param classType the class to be checked
   */
  private boolean needsTransmission (Class classType) {

    if (classType.isArray()) {
      classType = classType.getComponentType();
    }

    if (classType.isPrimitive()) {
      return false;
    }
    String className = classType.getName();
    // No need to send java classes
    if (className.startsWith ("java.") || className.startsWith("magician.")) {
      return false;
    }
    // Don't send the ActiveNodeManager 
    // if (primitiveClasses.contains(className)) {
    //   return false;
    // }

    return true;
  }

}
