/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * @(#)Queue.java	1.4 96/11/23
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 * @version 1.4, 11/23/96
 * @author Herb Jellinek
 */

package magician.Node;

import java.util.Enumeration;
import java.util.NoSuchElementException;


public class SchedQueue {

    int length = 0;

    QueueElement head = null;
    QueueElement tail = null;

    public SchedQueue() {
    }

    /**
     * Enqueue an object.
     * @param obj object to be queued
     */
    public synchronized void enqueue(Object obj) {

	QueueElement newElt = new QueueElement(obj);

	if (head == null) {
	    head = newElt;
	    tail = newElt;
	    length = 1;
	} else {
	    newElt.next = head;
	    head.prev = newElt;
	    head = newElt;
	    length++;
	}
	notifyAll();
    }

    /**
     * Dequeue the oldest object on the queue.  Will wait indefinitely.
     * @return the oldest object on the queue.
     * @exception InterruptedException if operation is interrupted
     */
    public  Object dequeue() throws InterruptedException {
	return dequeue(0L);
    }

    /**
     * Dequeue the oldest object on the queue.
     * @param timeOut the number of milliseconds to wait for something 
     * to arrive.
     * @return the oldest object on the queue.
     * @exception InterruptedException if operation is interrupted
     */
    public synchronized Object dequeue(long timeOut)
        throws InterruptedException {
	
	while (tail == null) {
	    wait(timeOut);
	}
	QueueElement elt = tail;
	tail = elt.prev;
	if (tail == null) {
	    head = null;
	} else {
	    tail.next = null;
	}
	length--;
        notifyAll();
	return elt.obj;
    }

    /**
     * Is the queue empty?
     * @return true if the queue is empty.
     */
    public boolean isEmpty() {
	return (tail == null);
    }

    /**
     * Returns an enumeration of the elements in Last-In, First-Out
     * order. Use the Enumeration methods on the returned object to
     * fetch the elements sequentially.
     * @return an enumeration of objects in the queue
     */
    public final synchronized Enumeration elements() {
	return new LIFOQueueEnumerator(this);
    }

    /**
     * Returns an enumeration of the elements in First-In, First-Out
     * order. Use the Enumeration methods on the returned object to
     * fetch the elements sequentially.
     * @return an enumeration of objects in the queue
     */
    public final synchronized Enumeration reverseElements() {
	return new FIFOQueueEnumerator(this);
    }

    /**
     * writes objects to output with message
     * @param msg the message to prepended to output
     */
    public synchronized void dump(String msg) {
	System.err.println(">> "+msg);
	System.err.println("["+length+" elt(s); head = "+
			   (head == null ? "null" : (head.obj)+"")+
			   " tail = "+(tail == null ? "null" : (tail.obj)+""));
	QueueElement cursor = head;
	QueueElement last = null;
	while (cursor != null) {
	    System.err.println("  "+cursor);
	    last = cursor;
	    cursor = cursor.next;
	}
	if (last != tail) {
	    System.err.println("  tail != last: "+tail+", "+last);
	}
	System.err.println("]");
    }
}


/**
 * class implementing a FIFO enumeration
 */
final class FIFOQueueEnumerator implements Enumeration {
    SchedQueue queue;
    QueueElement cursor;

    FIFOQueueEnumerator(SchedQueue q) {
	queue = q;
	cursor = q.tail;
    }

    public boolean hasMoreElements() {
	return (cursor != null);
    }

    public Object nextElement() {
	synchronized (queue) {
	    if (cursor != null) {
		QueueElement result = cursor;
		cursor = cursor.prev;
		return result.obj;
	    }
	}
	throw new NoSuchElementException("FIFOQueueEnumerator");
    }
}

/**
 * class implementiong LIFO enumeration
 */
final class LIFOQueueEnumerator implements Enumeration {
    SchedQueue queue;
    QueueElement cursor;

    LIFOQueueEnumerator(SchedQueue q) {
	queue = q;
	cursor = q.head;
    }

    public boolean hasMoreElements() {
	return (cursor != null);
    }

    public Object nextElement() {
	synchronized (queue) {
	    if (cursor != null) {
		QueueElement result = cursor;
		cursor = cursor.next;
		return result.obj;
	    }
	}
	throw new NoSuchElementException("LIFOQueueEnumerator");
    }
}

/**
 * class implementing description of queue element
 */
class QueueElement {
    QueueElement next = null;
    QueueElement prev = null;

    Object obj = null;

    QueueElement(Object obj) {
	this.obj = obj;
    }

    public String toString() {
	return "QueueElement[obj="+obj+(prev == null ? " null" : " prev")+
	    (next == null ? " null" : " next")+"]";
    }
}

