/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * Copyright (c) 1997 by the University of Kansas Center for Research, Inc.
 * All Rights Reserved.  This software was developed by the Information Technology
 * and Telecommunication Center (ITTC) at the University of Kansas
 * with the assistance of DARPA.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for personal or educational use is hereby granted, provided
 * that copies of the software are not made or distributed for profit or
 * commercial advantage; that both the copyright notice and this permission
 * notice appear in all copies of the software, derivative works
 * or modified versions, and any portions thereof; that both notices appear in
 * supporting documentation; and that the use of this software is acknowledged
 * in any publications that might result from the use of this software.
 *
 * ITTC ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, AND
 * DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 *
 * RCS Identification : $Id: Scheduler.java,v 1.2 2000/11/03 15:02:48 kulkarni Exp kulkarni $
 *
 * @author Yulia Wijata, Amit B. Kulkarni
 */


package magician.Node;

import java.lang.*;
import java.util.*;
import magician.Utils.*;
import magician.PerfTools.*;

public class Scheduler implements PerfConstants {

  /**
   * maximum number of SmartPacket threads that can run concurrently
   */
  protected int maxThreads = 0;
  
  /**
   * the queue of ready Smartpacket threads
   */
  protected SchedQueue readyQueue;
  
  /**
   * list of running SmartPacket threads 
   */
  protected UnsynchVector runningThreads;

  /**
   * Event manager to log events
   */
  EventManager  EventMgr;

  /**
   * table to maintain queueng times
   */
  protected Hashtable QTime;

  /**
   * the clock thread
   */
  static Thread ticker;

  /**
   * index of oldest executing Smartpacket thread
   */
  private int oldest = 0;

  /**
  * initializes the scheduler with the maximum number of runnable threads
  * @param maxThreads number of threads
  */
  public Scheduler(int maxThreads) {
    readyQueue = new SchedQueue();
    QTime = new Hashtable();
    EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");
    this.maxThreads = maxThreads;
    runningThreads = new UnsynchVector(maxThreads);
    ticker = new tickThread(this, 1000);
    ticker.setPriority(Thread.MAX_PRIORITY);
    ticker.start();
  }

  /**
   * checks on active threads and performs thread maintenance
   */
  public void tick () {
  
    SPThread t;
    // System.err.println("Scheduler: invoked at time " + System.currentTimeMillis());
    long now = System.currentTimeMillis();
    String sbuf = "VALUE=" + String.valueOf(now);
    EventMgr.postEvent(TR_USER, "ActiveNode.CurrentTime", now, 0, sbuf);

    int runners = runningThreads.size();

    // for (int i=0; i<runningThreads.size(); i++) {
    for (Enumeration thrs = runningThreads.elements(); thrs.hasMoreElements();) {
      // SPThread sp = (SPThread)runningThreads.elementAt(i);
      SPThread sp = (SPThread) thrs.nextElement();
      
      try {
        // if (!((SPThread)runningThreads.elementAt(i)).isAlive()) {
        if (!sp.isAlive()) {
          // remove dead thread
          if (EventMgr.traceDepthSupported(TR_PACKET)) {
          EventMgr.postEvent(TR_PACKET, "ResourceManager.SPThread.Completed", System.currentTimeMillis(), sp.getSmartPacketforSPThread().PacketID);
          }
	  // runningThreads.removeElementAt(i);
          runningThreads.removeElement(sp);

	  if (!readyQueue.isEmpty()){
	    t = (SPThread)readyQueue.dequeue();
            logQueueingTime(t);
	    runningThreads.addElement(t);
            /** 
             * reduce priority of thread to allow tick thread to intervene
             */
            //t.setPriority(t.getPriority() - 1);
            // System.err.println("Scheduler: Starting thread from readyqueue");
	    t.start();
	  }
        }
       } catch (InterruptedException e) {
          e.printStackTrace();
       } catch (NullPointerException e) {
	 // System.err.println("Scheduler: Thread not alive");
          continue;
      }
    }  
  }

  /**
   * logs queuing time for given thread
   * @param t the SmartPacket thread
   */
  private void logQueueingTime(SPThread t) {
    
    long timeQueued = ((Long) QTime.get(t)).longValue();
    // remove the entry from the table
    QTime.remove(t);
    if (EventMgr.traceDepthSupported(TR_MEASURE)) {
    String sbuf = "DELAY=" + String.valueOf(System.currentTimeMillis()-timeQueued);
    EventMgr.postEvent(TR_MEASURE, "ResourceManager.SPThread.QueueingDelay", System.currentTimeMillis() , t.getSmartPacketforSPThread().PacketID, sbuf);
    } 
  }

  /**
   * invokes the Schduler to remove the calling Smartpacket thread 
   */
  protected static void invoke() {
    SPThread spt = SPThread.currentSPThread();
    spt.removeSPThread();
    ticker.interrupt();
  }

  /**
   * Schedules the Smartpacket thread for execution
   * @param t the Smartpacket thread
   */
  public void Schedule(SPThread t) {
    // t.setPriority(Thread.MAX_PRIORITY - 1);
 
    // If the number of runningThreads reaches the max, 
    // queue this thread to run later else start it
    int nThreads = runningThreads.size();
    if (nThreads >= maxThreads) {
      
    if (EventMgr.traceDepthSupported(TR_PACKET)) {
    EventMgr.postEvent(TR_PACKET, "ResourceManager.SPThread.ThreadQueued", System.currentTimeMillis() , t.getSmartPacketforSPThread().PacketID);
    } 
      
      System.err.println("Scheduler: " + nThreads + " running");
      readyQueue.enqueue(t);
      // add time queued to table under thread's key
      QTime.put(t, new Long(System.currentTimeMillis()));
    } else {
      runningThreads.addElement(t);
     
    if (EventMgr.traceDepthSupported(TR_PACKET)) {
        String sbuf = "CLASSNAME=" + t.getSmartPacketforSPThread().getClass().getName();
        EventMgr.postEvent(TR_PACKET, "ResourceManager.SPThread.ThreadStarted", System.currentTimeMillis(), t.getSmartPacketforSPThread().PacketID);
    } 
      
      t.start();
     // Thread.currentThread().yield();
    }
  }

}

/**
 * Class that implements the Scheduler clock
 */
class tickThread extends Thread {

  /**
   * pointer to invoker
   */
  protected Scheduler parent;

  /**
   * clock period
   */
  protected long period;

  /**
   * initializes the clock interval
   * @param o the Scheduler that invoked this object
   * @param period the clock period
   */
  public tickThread(Scheduler o, long period) {
     parent = o;
     this.period = period;
  }

  /**
  * runs periodically and invokes schedulers <tt>tick</tt> method
  */
  public void run() {
 
     for (;;) {
        try {
           this.sleep(period);
        } catch (InterruptedException e) {
	  // System.err.println("Interrupted!");
        }
        parent.tick();
     }
  }
}

