/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

/**
 *
 *
 * Implements the hop by hop, receiver side sliding window acknowledgement protocol.
 *
 * @see ReliableCommFW
 */
package magician.Node;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.*;
import magician.PerfTools.*;

public class SliWinAck implements Serializable, PerfConstants {

  /**
   * node to be acknowledged
   */
  String Ackee;

  /**
   * Node from which ack is sent
   */
  transient String Acker;

  /**
   * sequence number of current packet
   */
  int seqNo;

  final static int WINDOW_SIZE = 8;

  /**
    * Event manager to report on activities of this module
    */
  transient EventManager EventMgr;


  /**
  * contructor
  */
  public SliWinAck () {
    Acker = ActiveNodeManager.GetNodeName();
  }

  /**
   * interface for user to send ack back to receiving node
   * @param typeId the type of the Smartpacket
   * @exception IndexOutOfBoundsException if sequence number is out of order
   */
  public void sendAck(byte[] typeId) throws IndexOutOfBoundsException {
    EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");

    if (Ackee == null || Ackee.equals(ActiveNodeManager.GetNodeName())) {
      // System.err.println("StopAndWaitAck: packet created here, no ack reqd");
      return;
    }

    // System.err.println("SliWinAck: Packet " + seqNo + " at time " + System.currentTimeMillis());
    /**
     * hack to prevent transmissions of class files back to sender
     */
    OnTheWirePacket.getLoader(typeId).setNodeReached(Ackee);

    String seqCache = "__SAWAP." + new String(typeId) + "." + Ackee; 
    StringTokenizer st = null;
    // repeat value tells us if we have received AND processed this packet before
    // and are accepting this packet because we rolled back for ack purposes
    boolean repeat = false;

    // check if we need to send Ack
    Vector buf = null;
    
    try {
      buf = (Vector) ActiveNodeManager.GetSmallState(seqCache);
      // System.err.println("SliWinAck: seq " + seqNo + " Obtained window value = " + buf + " at time " + System.currentTimeMillis());

      Integer Iseq = new Integer(seqNo);
      boolean seqInWindow = buf.contains(Iseq);

      if (EventMgr.traceDepthSupported(TR_PACKET)) {
        String sbuf = "EXPECTED=" + Ackee+"_"+buf + " SEQNO=" + String.valueOf(seqNo);
        EventMgr.postEvent(TR_PACKET, "KU_SmartPacket.SliWinAck.Status", System.currentTimeMillis(), 0, sbuf);
      }
      if (! seqInWindow) { // we are probably receiving a packet that has been
        // acknowledged before
        ActiveNodeManager.SetSmallState(seqCache, buf);
        repeat = true;
      } else {
        // seqNo as expected
        buf.removeElement(Iseq);
        // slide receive window to include next highest seq no
        if (buf.size() > 0) {
          int highSeq = ((Integer) buf.lastElement()).intValue();
          buf.addElement(new Integer(highSeq+1));
        } else {
          buf.addElement(new Integer(seqNo+1));
        }
        ActiveNodeManager.SetSmallState(seqCache, buf);
      }
    } catch (StateNotFoundException e) {
      // implies no small state exists
      try {
        ActiveNodeManager.CreateSmallState(seqCache);
        buf = new Vector(WINDOW_SIZE);
        for (int ix = 0; ix < WINDOW_SIZE; ix++) {
           buf.addElement(new Integer(ix+2));
        }
        if (EventMgr.traceDepthSupported(TR_PACKET)) {
          String sbuf = "EXPECTED=" + Ackee+"_"+buf + " SEQNO=" + String.valueOf(seqNo);
          EventMgr.postEvent(TR_PACKET, "KU_SmartPacket.SliWinAck.Status", System.currentTimeMillis(), 0, sbuf);
        }

        ActiveNodeManager.SetSmallState(seqCache, buf);
        // System.err.println("SliWinAck: Created new state at " + seqCache + " value = " + buf);
      } catch (Exception se) {
        // cannot occur
        se.printStackTrace();
        // if it does, this packet is a duplicate
        SPThread.currentThread().yield();
      }
    } catch (StateEmptyException e) {
      System.err.println("SliWinAck: State empty");
      throw new IndexOutOfBoundsException();
      // e.printStackTrace();
    }
    

    // send Acknowledgement packet to Ackee
    Acknowledgement Ack = AckManager.getAckPacket();

    Acker = ActiveNodeManager.GetNodeName();

    Ack.sendTo(Ackee);
    // Ack.sentFrom(Acker);
    Ack.setMailboxParams(seqNo, new String(typeId));
   
    ActiveNodeManager.ProcessSP(Ackee, Ack);

    // Ack sent, now check if this packet was processed earlier, 
    // if so, discard packet
    if (repeat) {
       throw new IndexOutOfBoundsException();
    }

    // SPThread.currentSPThread().yield();
  }


  /** 
   * interface for user to send packet
   * @param PortName name of the next-hop active node
   * @param SPObject the SmartPacket object being sent
   */
  public void waitForAck(String PortName, ReliableCommFW SPObject) throws NoRouteToHostException {

    Ackee = ActiveNodeManager.GetNodeName();
    // System.err.println("SliWinAck: Packet " + SPObject.PacketID + " getting seq no");
    /** race condition is created between this and next call **/
    
    // seqNo = AckManager.getNextSequenceNumber(PortName, new String(SPObject.getTypeId()));

    AckManager.queueForTransmission(PortName, SPObject);
    
    // System.err.println("SliWinAck: Packet " + SPObject.PacketID + " received seq no");
    // try {
//       if (EventMgr.traceDepthSupported(TR_PACKET)) {
//       String sbuf = "SEQNO=" + String.valueOf(seqNo);
//       EventMgr.postEvent(TR_PACKET, "KU_SmartPacket.SliWinAck.SeqNum", System.currentTimeMillis(), SPObject.PacketID, sbuf);
//       }
//     } catch (NullPointerException npe) {
//       EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");
//       if (EventMgr.traceDepthSupported(TR_PACKET)) {
//       String sbuf = "SEQNO=" + String.valueOf(seqNo);
//       EventMgr.postEvent(TR_PACKET, "KU_SmartPacket.SliWinAck.SeqNum", System.currentTimeMillis(), SPObject.PacketID, sbuf);
//       }
//     }
  }

}
