/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * Copyright (c) 1997 by the University of Kansas Center for Research, Inc.
 * All Rights Reserved.  This software was developed by the Information Technology
 * and Telecommunication Center (ITTC) at the University of Kansas
 * with the assistance of DARPA.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for personal or educational use is hereby granted, provided
 * that copies of the software are not made or distributed for profit or
 * commercial advantage; that both the copyright notice and this permission
 * notice appear in all copies of the software, derivative works
 * or modified versions, and any portions thereof; that both notices appear in
 * supporting documentation; and that the use of this software is acknowledged
 * in any publications that might result from the use of this software.
 *
 * ITTC ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, AND
 * DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE.
 *
 * RCS Identification : $Id: SliWinPortThread.java,v 1.1 2000/03/21 13:25:21 bushsf Exp kulkarni $
 *
 * @author Amit B. Kulkarni
 */


package magician.Node;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;
import magician.PerfTools.*;
import magician.Utils.*;

public class SliWinPortThread extends Thread implements PerfConstants {

  /**
   * name of port being managed
   */
  String PortName;

  /**
   * physical address of node
   */
  String ipAddress;

  /**
   * port number
   */
  int port;

  /**
   *  queue of outgoing packets
   */
  HashQueue outQ;

  /**
   * list of seqnos in window
   */
  Hashtable WinList;

  Queue retransThrs;
  
  /**
    * last seq number in window
    */
  int lastSeqNo = 1;

  EventManager EventMgr;

  
  final static int MAX_RETRIES = 6;
  final static int WINDOW_SIZE = 8;
  int speed = 0;
 
  
  public SliWinPortThread (Hashtable Port) {
    PortName = (String) Port.get("node.name");
    port = Integer.decode((String) Port.get("node.port")).intValue();
    ipAddress = (String) Port.get("node.LLAddress");
    speed = Integer.decode((String) Port.get("node.linkBandwidth")).intValue();
    outQ = new HashQueue();  
    WinList = new Hashtable();
    
    EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");
  }


  /**
   * queues the packet for transmission and retransmits
   * after timeout if ack is not received
   * Currently, individual timeout is ignored, only default is used.
   * @param seqNo the assigned sequence number for the packet
   * @param packet a byte array containing the serialized Smartpacket
   * @param timeout the timeout interval to use for retransmission
   */
  public void queueForTransmission(int seqNo, byte[] packet) throws NoRouteToHostException {
    try {
      if (EventMgr.traceDepthSupported(TR_PACKET)) {
      String sbuf = "NODE=" + PortName + " SEQNO=" + String.valueOf(seqNo) + " PTQ.SIZE=" + String.valueOf(outQ.size());
      EventMgr.postEvent(TR_PACKET, "PortThread.Queue.Size", System.currentTimeMillis(), 0, sbuf);
      }
      outQ.insert(new Integer(seqNo), packet);
    } catch (NullPointerException npe) {
      throw new NoRouteToHostException();
    }
    return ;
  }    

  /**
   * Confirms the receipt of the transmitted packet and removes it from the
   * retransmit queue.
   * @param seqNo the sequence number of the packet being confirmed
   * @exception IndexOutOfBoundsException if sequence number is out of order
   */
  public boolean confirmPacket(int seqNo) {
    Integer IseqNo = new Integer(seqNo);

    RetransmitThread thr = (RetransmitThread) WinList.get(IseqNo);

    if (WinList.remove(IseqNo) == null) {
      return false;
    }

    if (EventMgr.traceDepthSupported(TR_PACKET)) {
      String sbuf = "NODE=" + PortName + " SEQNO=" + String.valueOf(seqNo); 
      EventMgr.postEvent(TR_PACKET,"ActiveNodeManager.PortThread.ConfirmAck", System.currentTimeMillis(), 0, sbuf);
    }

    thr.confirmPacket();
    retransThrs.insert(thr);
    // System.err.println("SliWinPortThread: Confirmed Ack for " + seqNo + " at time " + System.currentTimeMillis());
    return true;
  }

  protected synchronized void PortDown() {
    // clean up all queued entries
    outQ.clear();
    outQ = null;

    Enumeration e = WinList.elements();
    while (e.hasMoreElements()) {
      RetransmitThread thr = (RetransmitThread) e.nextElement();
      thr.interrupt();
    }
    // mark link as down and send PortDown message to Routing Mgr.
    System.err.println("PortThread: too many attempts to reach " + PortName);
    RoutingMgr.defaultRouting().PortDown(PortName);

    return;
  }

  /**
   * sends packet at the front of the queue to 
   * the neighbor
   */
  public void run() {
    // create WINDOW_SIZE number of Retransmit threads and put them in a stack
    retransThrs = new Queue(WINDOW_SIZE);
    for (int ithr = 0; ithr < WINDOW_SIZE; ithr++) {
       RetransmitThread thr = new RetransmitThread(this, ipAddress, port, speed, MAX_RETRIES);
       thr.setPriority(Thread.MAX_PRIORITY - 2); // set lower than Ack thread
       retransThrs.insert(thr);
       thr.start();
    }

    while (true) {
      try {
        Integer ILastSeq = new Integer(lastSeqNo);
        if (EventMgr.traceDepthSupported(TR_PACKET)) {
          String sbuf = "NODE=" + PortName + " SEQNO=" + String.valueOf(lastSeqNo); 
          EventMgr.postEvent(TR_PACKET,"PortThread.FetchPacket.SeqNo", System.currentTimeMillis(), 0, sbuf);
        }
        byte[] packet = (byte[]) outQ.fetch(ILastSeq);
        outQ.delete(ILastSeq);
        if (packet == null) {
           if (EventMgr.traceDepthSupported(TR_PACKET)) {
             String sbuf = "NODE=" + PortName + " SEQNO=" + String.valueOf(lastSeqNo); 
             EventMgr.postEvent(TR_PACKET, "PortThread.Dequeue.PacketNull", System.currentTimeMillis(), 0, sbuf);
           }
          lastSeqNo++;
          continue;
        }

        RetransmitThread thr = (RetransmitThread) retransThrs.remove();
        // System.err.println("SliWinPortThread: Fetching and sending seq# " + lastSeqNo + " size = " + packet.length + " at time " + System.currentTimeMillis());
        WinList.put(ILastSeq, thr);
        thr.setPacket(lastSeqNo, packet);
        lastSeqNo++;

      } catch (InterruptedException ie) {
        System.err.println(ie);
      }
    } 
  }
}
