/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

//
// SmartPacketLoader.java
//
// Handles the loading of classes for a virtual node.  All attempts to load
// or resolve classes from the network pass through here
//
package magician.Node;
import java.io.*;
import java.net.*;
import java.util.*;

/**
  * takes the
  * @param
  */
public class SmartPacketLoader extends ClassLoader {
  // Hash table for cached classes that have already been loaded
  protected Hashtable cache;

  /**
   * cache to hold class bytes
   */
  protected Hashtable byteCache;

  /**
   * list of nodes to which this type has been sent
   * prevents sending classes again to those nodes repeatedly
   */
  Vector NodesReached;

  /**
   * holds path prefixes
   */
  Vector PathPrefixes;

  // Input object stream for reading from network
  // SPObjectInputStream is;

  // Constructor for SmartPacketLoader
  protected SmartPacketLoader() {
    // call the superclass' constructor
    super();

    // get and store classpath
    String classpath = System.getProperty("java.class.path");
    StringTokenizer st = new StringTokenizer(classpath, ":");
    PathPrefixes = new Vector();
    while(st.hasMoreTokens()) {
      PathPrefixes.addElement(st.nextToken());
    }
    
    // initialize the hash tables
    cache = new Hashtable();
    byteCache = new Hashtable();
    NodesReached = new Vector();
    NodesReached.addElement(ActiveNodeManager.GetNodeName());
  }

   /* 
    * adds names of active nodes that are known to have 
    * class definitions
    * @param node name of active node
    */
  protected void setNodeReached(String node) {
    try {
      if (! NodesReached.contains(node))
        NodesReached.addElement(node);
    } catch (NullPointerException e) {
      // do nothing
    }
  }

   /* 
    * checks if given active nodes is known to have 
    * class definitions
    * @param node name of active node
    */
  public boolean checkNodeReached(String node) {
    try {
      if (NodesReached.contains(node))
        return true;
    } catch (NullPointerException e) {
      // do nothing
    }
    return false;
  }

  // Return the Class with the specified name
  public Class getClass (String name) throws ClassNotFoundException {
    Class c = (Class) cache.get (name);
    if (c == null) {
	c = Class.forName (name);
    }
    return c;
  }

  /*
    * takes the
    * @param
    */
  protected Class defClass(String name) {
    byte[] ClassData = null;

    Class c = null;
    //joust 
    //System.err.println("SmartPacketLoader: finding def class " + name);
    c = (Class) cache.get(name);
    if (c != null)
      return c;

    
    if ((ClassData = (byte[]) byteCache.get(name)) == null) 
      return null;
     

    try {
      //joust
      //System.err.println("SmartPacketLoader: defining class " + name);

      c = defineClass(name, ClassData, 0, ClassData.length);

      // If we found it and were able to define it, cache 
      // the class
      //joust
      // System.err.println("SmartPacketLoader: adding class " + name);
      cache.put(name, c);
    } catch (ClassFormatError e) {
      try {
        c = Class.forName(name);
      } catch (ClassNotFoundException cnf) {
        cnf.printStackTrace();
      }
    } catch (Exception e) {	
	//System.err.println("SmartPacketLoader: Exception " + e + ": for class " + name);
      try {
        c = Class.forName(name);
      } catch (ClassNotFoundException cnf) {
        cnf.printStackTrace();
      }
    }
 
    return c;
  }

  /*
    * takes the
    * @param
    */
  protected  void makeClass(String name, byte[] data) {
      //System.err.print("SmartPacketLoader: Checking class " + name + "...");
      if (byteCache.get(name) == null) {
        byteCache.put(name, data);
      }

      //System.err.println("exists");

   /*******
    Class c = null;
    try {
      // System.err.println("SmartPacketLoader: defining class " + name);

      c = defineClass(name, data, 0, data.length);
      resolveClass(c);
      cache.put(name, c);
    } catch (ClassFormatError e) {
      c = (Class) cache.get(name);
    } catch (Exception e) {
      try {
        c = Class.forName(name);
      } catch (ClassNotFoundException cnf) {
        cnf.printStackTrace();
      }
    }
    *******/
  }

  /*
    * takes the
    * @param
    */
  protected byte[] localLoadClass(String ClassName, String toNode) 
    throws ClassNotFoundException, CloneNotSupportedException {
    try {
      if (NodesReached.contains(toNode)) {
        throw new CloneNotSupportedException();
      }
    } catch (NullPointerException e) {
      // to Node must be null
      throw new CloneNotSupportedException();
    }

    return this.localLoadClass(ClassName);
    }


  // Used to load classes that are on the local filesystem
  protected  byte[] localLoadClass (String className) throws ClassNotFoundException {
    // search for class in cache
    if (byteCache.containsKey(className)) {
      // System.err.println("SmartPacketLoader: Classbytes for class " + className + " in cache");
      return (byte[]) byteCache.get(className);
    }
    
    // else open class file
    File fd = null;
    try {
       String classname = className.replace('.', '/');
      fd = searchForFile(classname + ".class");
    } catch (FileNotFoundException e) {
       throw new ClassNotFoundException();
    }
    
    FileInputStream fis = null;
    try {                       
      fis = new FileInputStream(fd);
    } catch (FileNotFoundException e) {
      return null;
    }
    
    // Read the class data
    int length = 0;
    byte[] classDesc = null;
    try {
      length = fis.available();
      classDesc = new byte[length];
      fis.read(classDesc, 0, length);
    } catch (IOException e) {
      return null;
    }
    // Define  the class
    Class c = null;
    try {
      c = defineClass(className, classDesc, 0, length);
      cache.put(className, c);
    } catch (Exception e) {
      try {
        c = Class.forName(className);
      } catch (ClassNotFoundException cnf) {
        cnf.printStackTrace();
      }
    } catch (ClassFormatError cfe) {
      c = (Class) cache.get(className);
    } catch (LinkageError le) {
      c = (Class) cache.get(className);
    }
        
    byteCache.put(className, classDesc);

    return classDesc;
  }


  // Load a Class given its name.  
  protected Class loadClass(String name, boolean resolve) {
     //joust;uncommented
     // System.err.println("loadClass: loading class " + name);

    Class c = null;

    // Check if we've already cached the class
    if (cache.containsKey(name)) {
       c = (Class) cache.get(name);
    }


    if (c == null) {
      if (name.startsWith("magician")) {
        try {
          // System.err.println("SmartPacketLoader: loading class " + name + " from default loader");
          c = findSystemClass(name);
           // System.err.println("SmartPacketLoader: loaded " + c + " from default loader");
          cache.put(name, c);
          return c;
        } catch (Exception me) {
	   //joust; uncommented
           System.err.println("SmartPacketLoader: not loaded " + name + " from default loader");
        }
      } else if (name.startsWith("java")) {
         // Check if it's a system class
         try {
	   c = findSystemClass(name);
	   cache.put(name, c);
         } catch (Exception e) {
           e.printStackTrace();
         } 
      } else if (name.startsWith("[")) {
        // An array, find the class for it
        try {
          if ((c = Class.forName(name)) != null) {
            cache.put(name,c);
	  }
        } catch (Exception e) {
          e.printStackTrace();
        }
      } else {
        if ((c = defClass(name)) == null) {
          try {
            c = Class.forName(name);
            cache.put(name, c);
          } catch (Exception t) {
            System.err.println("SmartPacketLoader: Could not resolve class " + name);
          }
        }
      }
    }

    // If we need to resolve the class, call resolveClass on it
    if (c != null && !name.startsWith("[") && resolve) {
      // System.err.println("SmartPacketLoader: resolving class " + c.getName());
      resolveClass(c); 
    }

    // System.err.println("SmartPacketLoader: loaded and resolved class " + c.getName());
    return (c);
  }

  /**
   * appends class path prefixes to search for file 
   *
   * @param filename the name of the file
   * @returns the file descriptor of the file or null
   * @Exception FileNotFoundException
   */
  private File searchForFile(String filename) throws FileNotFoundException {
    File fd = null;

    Enumeration prefix = PathPrefixes.elements();
    // prepend every path prefix 
    do {
      String newClassFile = prefix.nextElement() + "/" + filename;    
 
      fd = new File(newClassFile);
    } while(!fd.exists() && prefix.hasMoreElements());
   
    return fd;
  }
}
