/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/


package magician.Node;

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * The TableEntry is a Smartpacket extension used by the RIPRouting protocol. 
 * Each one has a 
 * the information about one entry in the router table and is sent to neighboring nodes.
 * Each entry is a 4-tuple containing information about a network node known by its 
 * virtual name, the next hop to be used for forwarding to the destination, the hop 
 * count to the node and the update interval for the entry. If the entry is not updated
 * within the given interval, it is automatically removed.
 *
 * @see RIPRouting
 */
public class TableEntry extends  KUSmartPacketV2 {


  /**
   * address of the remote node represented by this entry
   */
   String         laddr;                //destination address

  /**
   * address of the node to forward packet to get to the destination
   */
   String         first_gateway;        //next hop to dest

  /**
   * the hop distance of the destination node
   */
   int            metric;              //number of hops

  /**
   * update/timeout interval for this entry
   */
   int 	 	  timer;               //falg to check last update


   /**
    * class constructor
    */
   public TableEntry () {
       CPU_Requirements = 50;
   }


     /**
      * checks and updates entry belonging to the active node that sent this
      */
     public void exec() {
       try {
         // System.out.println("Table Entry packet from " + Source_Address + " : metric =" + metric);
       
         RIPRouting.CheckEntry(this);
         
         //call the default router method to check received entry
         //The next thing is to discard this packet. (this smart packet 
         //destroys itself after it is done)
         halt();

       } catch (LinkageError l) {
          l.printStackTrace();
       } catch (Exception e) {
          e.getMessage();
          e.printStackTrace();
       }
     }

  /**
   * getnexthop returns the firstgateway address 
   * @return name of active node
   */
   public String getnexthop() {
          return first_gateway;
   }

  /**
   * set the first gateway address 
   * @param hadr name of active node
   */
   public void setnexthop(String hadr) {
         first_gateway=hadr;
         return;
   }

  /**
   * get the number of hops to the destination
   * @return number of hops
   */
   public int getmetric() {
          return metric;
   }

  /**
   * set the value of the number of hops to dest
   * @param met number of hops
   */
   public void setmetric(int met) {
         metric = met;
         return;
   }

   /** 
    * get the value of timer (when was it last updated)
    * @return timer value
    */
   public int gettimer() {
          return timer;
   }

  /**
   * set the value of the timer
   * @param fl the timer value
   */
   public void settimer(int fl) {
          timer=fl;
          return;
   }

  /**
   *  print contents
   */
   public void printDetails() {
 
   //MySystem.println("table entry:" + laddr+" "+getnexthop());
   //MySystem.println("metric and timer : "+getmetric()+" "+gettimer());
   //MySystem.println("source : " + Source_Address);
   }

}
