/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

package magician.PerfTools;

import java.lang.*;


/**
 * CircularList is implemented by maintaining 2 pointers, <tt>get<tt> and <tt>put<tt>
 * which point to the next allocated and free nodes on the list, respectively. The class
 * also provides interfaces to insert, remove and print its contents. 
 */
 
 public class CircularList {
   private Node get;            // points to the first allocated node
   private Node put;            // points to the first free node
   private int size;               // initial number of nodes on the list
   private Class objClass;  // the class of the object being stored
   /**
    * takes class of object to be created and the size of list 
    * @param objClass create nodes of instances of this class
    * @param size    initial size of list
    */
   public CircularList(Class objClass, int size) {
     this.objClass = objClass;
     this.size = size;
     get = put = null;
     
     try {
       int count = 0;
       while (count < size) {
          insert(objClass.newInstance());
          count++;
       }
       put = get;   // reset pointers back to head of list
     } catch (Throwable t) {
        System.err.println("CircularList.java(constructor): cannot create instance of class " + objClass.getName());
     }
   }
   
   /**
    * creates an empty circular list
    */
   public CircularList() {
      get = put = null;
      size = 0;
   }
   
   /**
    * insert into list
    */
   public void insert(Object obj)  {
      // create new node for the data
	  Node newNode = new Node(obj, get);
		
	  // if list is empty, set inserted object to head of list
	  if (get == null) {
		 get = put = newNode;
                 newNode.setNext(get);
	  } 
	  else {
		 put = put.setNext(newNode);
	  }
	}
	
	
	/**
	 * returns the object being updated 
	 */
	public Object set() throws NullPointerException {
	   if (put.next() == get) {
              throw new NullPointerException();
	   }
	   
	   Object o = put.get();
	   put = put.next();
	    
	   return o;
	}
	 
	/**
	 * returns the object info and frees it
	 */
	public Object get() {
	   if (get == put) return null;
	   
	   Object o = get.get();
	   get = get.next();
	   
	   return o;
	} 
}
