/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/

package magician.PerfTools;

import java.lang.*;
import java.io.*;
import java.util.*;
import magician.Node.ActiveNodeManager;

/**
 * Class NetLoggerEvent is a  event manager class derived from <tt>EventManager</tt>. This event manager is used when recording and printing event information in NetLogger format.
 *
 * @see EventManager
 * @see NetLoggerEvent
 */
 
public class NetLoggerEventManager implements NetEventListener {
  /*
   * name of host where logging occurs
   */
    protected  String HostName; 
    
  /*
   * output stream to write event information to
   */
   protected DataOutputStream out;

  /*
   * Eventmanager object to register events
   */
   protected EventManager EventMgr;

  /**
   * constructor starts a timer with the given interval and
   * sets up the output stream.
   * @param bufferFlushInterval  the time interval after which buffer are flushed to output
   * @exception  IllegalArgumentException thrown if  the size of the list is not a positive number
   * @exception  ClassNotFoundException  thrown if the class definition does not exist
   */
   public NetLoggerEventManager(long bufferFlushInterval, DataOutputStream out) throws IllegalArgumentException, ClassNotFoundException {
     this.out = out;
     EventMgr = (EventManager) ActiveNodeManager.HaveService("EventManager");  
     if (!EventMgr.flushBuffers(bufferFlushInterval)) {
        System.err.println("Buffer flushing was not enabled");
     }
   }

  /**
   * posts NetLogger event information. 
   * @param e NetEvent object describing the event 
   */
  public synchronized void postEvent(NetEvent e) {
      String eventDescriptor = e.getEventType();
      long timeStamp = e.getEventTime();
      long timeStampSec = timeStamp / 1000;  // since timeStamp is in milliseconds
      long timeStampUsec = (timeStamp - timeStampSec*1000) * 1000;  // get remaining value as  microseconds
      HostName = ActiveNodeManager.GetNodeName();
      long PacketID = e.getPacketID();
      String extraFields = e.getOtherEventFields();
      String event = "NL.EVNT=" + eventDescriptor + " HOST=" + HostName + " DATE=" + timeStampSec + " NL.SEC=" + timeStampSec + " NL.USEC=" + timeStampUsec + " AN.PKT_ID=" + PacketID + " " + extraFields + "\n";
      try {
        out.writeInt(event.length());
        out.write(event.getBytes());
      } catch (IOException ioe) {
        ioe.printStackTrace();
      }
  }
}



