/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * @(#)ByteArrayDataOutStream.java	
 * 
 */


package magician.Utils;

import java.io.*;


public
class ByteArrayDataOutStream extends OutputStream implements DataOutput {

    /** 
     * The buffer where data is stored. 
     *
     * @since   JDK1.0
     */
    protected byte buf[];


    /**
     * The number of bytes written to the data output stream. 
     *
     * @since   JDK1.0
     */
    protected int written;

    /**
     * Creates a new data output stream to write data to the specified 
     * underlying output stream. 
     *
     * @param   out   the underlying output stream.
     * @see     java.io.FilterOutputStream#out
     * @since   JDK1.0
     */
    public ByteArrayDataOutStream() {
	this(32);
        written = 0;
    }

    /**
      * initializes internal buffer to given size
      * @param size size of buffer in bytes
      */
    public ByteArrayDataOutStream(int size) {
        buf = new byte[size];
        written = 0;
    }

    /**
     * Writes the complete contents of this byte array output stream to 
     * the specified output stream argument, as if by calling the output 
     * stream's write method using <code>out.write(buf, 0, count)</code>.
     *
     * @param      out   the output stream to which to write the data.
     * @exception  IOException  if an I/O error occurs.
     * @since      JDK1.0
     */
    public void writeTo(OutputStream out) throws IOException {
	out.write(buf, 0, written);
    }

    /**
     * Resets the <code>count</code> field of this byte array output 
     * stream to zero, so that all currently accumulated output in the 
     * ouput stream is discarded. The output stream can be used again, 
     * reusing the already allocated buffer space. 
     *
     * @see     java.io.ByteArrayInputStream#count
     * @since   JDK1.0
     */
    public void reset() {
	written = 0;
    }

    /**
     * Writes the specified byte to the underlying output stream. 
     *
     * @param      b   the <code>byte</code> to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public void write(int b) throws IOException {
	int newcount = written + 1;
	if (newcount > buf.length) {
	    byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
	    System.arraycopy(buf, 0, newbuf, 0, written);
	    buf = newbuf;
	}
	buf[written] = (byte)b;
	written = newcount;
    }

    /**
     * Writes <code>len</code> bytes from the specified byte array 
     * starting at offset <code>off</code> to the underlying output stream.
     *
     * @param      b     the data.
     * @param      off   the start offset in the data.
     * @param      len   the number of bytes to write.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public void write(byte b[], int off, int len)
	throws IOException
    {
	int newcount = written + len;
	if (newcount > buf.length) {
	    byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
	    System.arraycopy(buf, 0, newbuf, 0, written);
	    buf = newbuf;
	}
	System.arraycopy(b, off, buf, written, len);
	written = newcount;
    }

    /**
     * Writes a <code>boolean</code> to the underlying output stream as 
     * a 1-byte value. The value <code>true</code> is written out as the 
     * value <code>(byte)1</code>; the value <code>false</code> is 
     * written out as the value <code>(byte)0</code>.
     *
     * @param      v   a <code>boolean</code> value to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeBoolean(boolean v) throws IOException {
	this.write(v ? 1 : 0);
    }

    /**
     * Writes out a <code>byte</code> to the underlying output stream as 
     * a 1-byte value. 
     *
     * @param      v   a <code>byte</code> value to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeByte(int v) throws IOException {
	this.write(v);
    }

    /**
     * Writes a <code>short</code> to the underlying output stream as two
     * bytes, high byte first. 
     *
     * @param      v   a <code>short</code> to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeShort(int v) throws IOException {
	int newcount = written + 2;
	if (newcount > buf.length) {
	    byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
	    System.arraycopy(buf, 0, newbuf, 0, written);
	    buf = newbuf;
	}
	buf[written]   = (byte) ((v >>> 8) & 0xFF);
        buf[written+1] = (byte) ((v >>> 0) & 0xFF);
        written = newcount;
    }

    /**
     * Writes a <code>char</code> to the underlying output stream as a 
     * 2-byte value, high byte first. 
     *
     * @param      v   a <code>char</code> value to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeChar(int v) throws IOException {
	int newcount = written + 2;
	if (newcount > buf.length) {
	    byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
	    System.arraycopy(buf, 0, newbuf, 0, written);
	    buf = newbuf;
	}
	buf[written]   = (byte) ((v >>> 8) & 0xFF);
        buf[written+1] = (byte) ((v >>> 0) & 0xFF);
        written = newcount;
    }

    /**
     * Writes an <code>int</code> to the underlying output stream as four
     * bytes, high byte first. 
     *
     * @param      v   an <code>int</code> to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeInt(int v) throws IOException {
	int newcount = written + 4;
	if (newcount > buf.length) {
	    byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
	    System.arraycopy(buf, 0, newbuf, 0, written);
	    buf = newbuf;
	}
        buf[written]   = (byte) ((v >>> 24) & 0xFF);
        buf[written+1] = (byte) ((v >>> 16) & 0xFF);
	buf[written+2] = (byte) ((v >>> 8) & 0xFF);
        buf[written+3] = (byte) ((v >>> 0) & 0xFF);
        written = newcount;
    }

    /**
     * Writes a <code>long</code> to the underlying output stream as eight
     * bytes, high byte first. 
     *
     * @param      v   a <code>long</code> to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeLong(long v) throws IOException {
	int newcount = written + 8;
	if (newcount > buf.length) {
	    byte newbuf[] = new byte[Math.max(buf.length << 1, newcount)];
	    System.arraycopy(buf, 0, newbuf, 0, written);
	    buf = newbuf;
	}
	buf[written]   = (byte) ((v >>> 56) & 0xFF);
	buf[written+1] = (byte) ((v >>> 48) & 0xFF);
	buf[written+2] = (byte) ((v >>> 40) & 0xFF);
	buf[written+3] = (byte) ((v >>> 32) & 0xFF);
        buf[written+4] = (byte) ((v >>> 24) & 0xFF);
        buf[written+5] = (byte) ((v >>> 16) & 0xFF);
	buf[written+6] = (byte) ((v >>> 8) & 0xFF);
        buf[written+7] = (byte) ((v >>> 0) & 0xFF);
        written = newcount;
    }

    /**
     * Converts the float argument to an <code>int</code> using the 
     * <code>floatToIntBits</code> method in class <code>Float</code>, 
     * and then writes that <code>int</code> value to the underlying 
     * output stream as a 4-byte quantity, high byte first. 
     *
     * @param      v   a <code>float</code> value to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @see        java.lang.Float#floatToIntBits(float)
     * @since      JDK1.0
     */
    public final void writeFloat(float v) throws IOException {
	this.writeInt(Float.floatToIntBits(v));
    }

    /**
     * Converts the double argument to a <code>long</code> using the 
     * <code>doubleToLongBits</code> method in class <code>Double</code>, 
     * and then writes that <code>long</code> value to the underlying 
     * output stream as an 8-byte quantity, high byte first. 
     *
     * @param      v   a <code>double</code> value to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @see        java.lang.Double#doubleToLongBits(double)
     * @since      JDK1.0
     */
    public final void writeDouble(double v) throws IOException {
	this.writeLong(Double.doubleToLongBits(v));
    }

    /**
     * Writes out the string to the underlying output stream as a 
     * sequence of bytes. Each character in the string is written out, in 
     * sequence, by discarding its high eight bits. 
     *
     * @param      s   a string of bytes to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeBytes(String s) throws IOException {
	int len = s.length();
	for (int i = 0 ; i < len ; i++) {
	   this.write((byte)s.charAt(i));
	}
    }

    /**
     * Writes a string to the underlying output stream as a sequence of 
     * characters. Each character is written to the data output stream as 
     * if by the <code>writeChar</code> method. 
     *
     * @param      s   a <code>String</code> value to be written.
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.DataOutputStream#writeChar(int)
     * @see        java.io.FilterOutputStream#out
     * @since      JDK1.0
     */
    public final void writeChars(String s) throws IOException {
	int len = s.length();
	for (int i = 0 ; i < len ; i++) {
	    int v = s.charAt(i);
            this.writeChar(v);
	}
    }

    /**
     * Writes a string to the underlying output stream using UTF-8 
     * encoding in a machine-independent manner. 
     * <p>
     * First, two bytes are written to the output stream as if by the 
     * <code>writeShort</code> method giving the number of bytes to 
     * follow. This value is the number of bytes actually written out, 
     * not the length of the string. Following the length, each character 
     * of the string is output, in sequence, using the UTF-8 encoding 
     * for the character. 
     *
     * @param      str   a string to be written.
     * @exception  IOException  if an I/O error occurs.
     * @since      JDK1.0
     */
    public final void writeUTF(String str) throws IOException {
	int strlen = str.length();
	int utflen = 0;

	for (int i = 0 ; i < strlen ; i++) {
	    int c = str.charAt(i);
	    if ((c >= 0x0001) && (c <= 0x007F)) {
		utflen++;
	    } else if (c > 0x07FF) {
		utflen += 3;
	    } else {
		utflen += 2;
	    }
	}

	if (utflen > 65535)
	    throw new UTFDataFormatException();		  

        this.writeShort(utflen);
	for (int i = 0 ; i < strlen ; i++) {
	    int c = str.charAt(i);
	    if ((c >= 0x0001) && (c <= 0x007F)) {
		this.write(c);
	    } else if (c > 0x07FF) {
		this.write(0xE0 | ((c >> 12) & 0x0F));
		this.write(0x80 | ((c >>  6) & 0x3F));
		this.write(0x80 | ((c >>  0) & 0x3F));
	    } else {
		this.write(0xC0 | ((c >>  6) & 0x1F));
		this.write(0x80 | ((c >>  0) & 0x3F));
	    }
	}
    }

    /**
     * Creates a newly allocated string. Its size is the current size of 
     * this output stream and the valid contents of the buffer have been 
     * copied into it. Each character <i>c </i>in the resulting string is 
     * constructed from the corresponding element <i>b</i> in the byte 
     * array such that:
     * <ul><code>
     *     c == (char)(b &amp; 0xff)
     * </code></ul>
     *
     * @return  the current contents of this output stream, as a string.
     * @see     java.io.ByteArrayOutputStream#size()
     * @since   JDK1.0
     */
    public String toString() {
	return new String(toByteArray());
    }

    /**
     * Returns the number of bytes written to this data output stream.
     *
     * @return  the value of the <code>written</code> field.
     * @see     java.io.DataOutputStream#written
     * @since   JDK1.0
     */
    public final int size() {
	return written;
    }

    /**
     * Creates a newly allocated byte array. Its size is the current 
     * size of this output stream and the valid contents of the buffer 
     * have been copied into it. 
     *
     * @return  the current contents of this output stream, as a byte array.
     * @see     java.io.ByteArrayOutputStream#size()
     * @since   JDK1.0
     */
    public byte toByteArray()[] {
       /**
	* byte newbuf[] = new byte[written];
	* System.arraycopy(buf, 0, newbuf, 0, written);
        *
	* return newbuf;
        */
        return buf;
    }

}
