/**

Copyright ( 2001 The University of Kansas
COPYRIGHT AND LIMITATION ON USE
The documents and software (collectively the "Information") developed by the University of Kansas, Information and Telecommunication Technology Center (Hereinafter referred to as KU) is the property of KU or its licensors and is protected by copyright and other intellectual property laws. The information may be viewed, copied, run, modified or printed for educational and research purposes only and shall not be published, sold, broadcast, or redistributed, either directly or indirectly, in whole or in part, by any means or methods for commercial purposes. The Information may not be used for any purpose not expressly approved herein. Uses not approved include, but are not limited to, commercial distribution, in whole or in part.  Any rights not expressly granted herein are reserved by KU. 

Nothing contained herein shall be construed as conferring by implication, estoppel or otherwise any license or right under any patent or trademark of KU or any third party. Except as expressly provided above, nothing contained herein shall be construed as conferring any license or right under any KU copyright. 

DISCLAIMER OF WARRANTY
Use of the  information is at your sole risk. The information, is provided "AS IS" and without warranty of any kind and KU  AND ITS AFFILIATES, AGENTS AND LICENSORS , EXPRESSLY DISCLAIM ALL WARRANTIES, EXPRESS or IMPLIED, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND OF FITNESS FOR A PARTICULAR PURPOSE AND ANY EXPRESS OR IMPLIED WARRANTY AGAINST INFRINGEMENT. KU DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS, OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE SOFTWARE WILL BE CORRECTED. FURTHERMORE, KU DOES NOT WARRANT OR MAKE ANY REPRESENTATIONS REGARDING THE USE OR THE RESULTS OF THE USE OF THE SOFTWARE OR ITS DOCUMENTATION IN TERMS OF THEIR CORRECTNESS, ACCURACY, RELIABILITY, OR OTHERWISE. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY KU OR A KU AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY INCREASE THE SCOPE OF THIS WARRANTY. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU (AND NOT KU), ASSUMES THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION. 

LIMITATION OF LIABILITY

NEITHER KU NOR ANY OF ITS AFFILIATES, AGENTS OR REPRESENTATIVES WILL BE LIABLE FOR ACTUAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES (INCLUDING LOSS OF PROFITS, LOSS OF BUSINESS OR SAVINGS), EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL KU BE LIABLE FOR ANY ACTUAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER, WHETHER IN AN ACTION OF CONTRACT, TORT OR ACTION OF ANY KIND, ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE INFORMATION.  KU, ITS AFFILIATES, AGENTS AND REPRESENTATIVES WILL NOT BE LIABLE FOR NEGLIGENCE, RECKLESSNESS, INTENTIONAL MISCONDUCT OR ANY OTHER ACTS OR OMISSIONS OF THIRD PARTIES. 
GOVERNING LAW
This Agreement, all intellectual property issues, and your rights and obligations shall be governed by the laws of the United States of America and the State of Kansas, excluding their conflicts of laws rules. 


**/
/*
 * @(#)UnsynchByteArrayInputStream.java	1.21 97/01/22
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

package magician.Utils;

import java.io.*;

public
class UnsynchByteArrayInputStream extends InputStream {
    /**
     * The byte array containing the data. 
     *
     * @since   JDK1.0
     */
    protected byte buf[];

    /**
     * The index of the next character to read from the input stream buffer.
     *
     * @since   JDK1.0
     */
    protected int pos;

    /**
     * The currently marked position in the stream.
     * ByteArrayInputStreams are marked at position zero by
     * default when constructed.  They may be marked at another
     * position within the buffer by the <code>mark()</code> method.
     * The current buffer position is set to this point by the
     * <code>reset()</code> method.
     *
     * @since   JDK1.1
     */
    protected int mark = 0;

    /**
     * The index one greater than the last valid character in the input 
     * stream buffer. 
     *
     * @since   JDK1.0
     */
    protected int count;

    /**
     * Creates a new byte array input stream that reads data from the 
     * specified byte array. The byte array is not copied. 
     *
     * @param   buf   the input buffer.
     * @since   JDK1.0
     */
    public UnsynchByteArrayInputStream(byte buf[]) {
	this.buf = buf;
        this.pos = 0;
	this.count = buf.length;
    }

    /**
     * Creates a new byte array input stream that reads data from the 
     * specified byte array. Up to <code>length</code> characters are to 
     * be read from the byte array, starting at the indicated offset. 
     * <p>
     * The byte array is not copied. 
     *
     * @param   buf      the input buffer.
     * @param   offset   the offset in the buffer of the first byte to read.
     * @param   length   the maximum number of bytes to read from the buffer.
     * @since   JDK1.0
     */
    public UnsynchByteArrayInputStream(byte buf[], int offset, int length) {
	this.buf = buf;
        this.pos = offset;
	this.count = Math.min(offset + length, buf.length);
    }

    /**
     * Reads the next byte of data from this input stream. The value 
     * byte is returned as an <code>int</code> in the range 
     * <code>0</code> to <code>255</code>. If no byte is available 
     * because the end of the stream has been reached, the value 
     * <code>-1</code> is returned. 
     * <p>
     * The <code>read</code> method of <code>ByteArrayInputStream</code> 
     * cannot block. 
     *
     * @return  the next byte of data, or <code>-1</code> if the end of the
     *          stream has been reached.
     * @since   JDK1.0
     */
    public int read() {
	return (pos < count) ? (buf[pos++] & 0xff) : -1;
    }

    /**
     * Reads up to <code>len</code> bytes of data into an array of bytes 
     * from this input stream. This <code>read</code> method cannot block. 
     *
     * @param   b     the buffer into which the data is read.
     * @param   off   the start offset of the data.
     * @param   len   the maximum number of bytes read.
     * @return  the total number of bytes read into the buffer, or
     *          <code>-1</code> if there is no more data because the end of
     *          the stream has been reached.
     * @since   JDK1.0
     */
    public int read(byte b[], int off, int len) {
	if (pos >= count) {
	    return -1;
	}
	if (pos + len > count) {
	    len = count - pos;
	}
	if (len <= 0) {
	    return 0;
	}
	System.arraycopy(buf, pos, b, off, len);
	pos += len;
	return len;
    }

    /**
     * Skips <code>n</code> bytes of input from this input stream. Fewer 
     * bytes might be skipped if the end of the input stream is reached. 
     *
     * @param   n   the number of bytes to be skipped.
     * @return  the actual number of bytes skipped.
     * @since   JDK1.0
     */
    public long skip(long n) {
	if (pos + n > count) {
	    n = count - pos;
	}
	if (n < 0) {
	    return 0;
	}
	pos += n;
	return n;
    }

    /**
     * Returns the number of bytes that can be read from this input 
     * stream without blocking. 
     * <p>
     * The <code>available</code> method of 
     * <code>ByteArrayInputStream</code> returns the value of 
     * <code>count&nbsp;- pos</code>, 
     * which is the number of bytes remaining to be read from the input buffer.
     *
     * @return  the number of bytes that can be read from the input stream
     *          without blocking.
     * @since   JDK1.0
     */
    public int available() {
	return count - pos;
    }

    /**
     * Tests if ByteArrayInputStream supports mark/reset.
     *
     * @since   JDK1.1
     */
    public boolean markSupported() {
	return true;
    }

    /**
     * Set the current marked position in the stream.
     * ByteArrayInputStreams are marked at position zero by
     * default when constructed.  They may be marked at another
     * position within the buffer by this method.
     *
     * @since   JDK1.1
     */
    public void mark(int markpos) {
	mark = pos;
    }

    /**
     * Resets the buffer to the marked position.  The marked position
     * is the beginning unless another position was marked.
     *
     * @since   JDK1.0
     */
    public void reset() {
	pos = mark;
    }
}
